/*= AMIQ I/Q Modulation Generator ===========================================*/
/* LabWindows/CVI 5.0.1 Instrument Driver                                    */
/* Original Release: September 1999                                          */
/* By:   Jan Sima                                                            */
/*                                                                           */
/* Should you have any technical questions please contact                    */
/* the hotline of Rohde & Schwarz Vertriebs-GmbH                             */
/* Rohde & Schwarz Support Center                                            */
/* e-mail: CustomerSupport@rsd.rohde-schwarz.com                             */
/*                                                                           */
/* Modification History:                                                     */
/*    See ChangeLog                                                          */
/*===========================================================================*/

#include <visa.h>
#include <limits.h>
#include <string.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#ifdef _CVI_
	#include <utility.h>
#endif
	
#include "rsamiq.h"

/*= DEFINES =================================================================*/
#define RSAMIQ_REVISION     "1.36, 08/2010" /*  Instrument driver revision */
#define BUFFER_SIZE         512L         /* File I/O buffer size */

#define AMIQ_DEFAULT_TIMEOUT    10000   /* Default AMIQ Operations Timeout value in ms (10 sec) */
#define AMIQ_ADJUSTMENT_TIMEOUT 100000  /* Calibration Timeout value in ms (100 sec) */
#define MAX_AMIQ_HD_FILE_LOAD_TIMEOUT   (file_size/speed+10000) /* TMO value in ms (calculated) */
#define MAX_AMIQ_FLOPPY_FILE_LOAD_TIMEOUT   (file_size/flopySpeed+10000)    /* TMO value in ms (calculated) */
#define MAX_AMIQ_MARKER_SHIFT_TIMEOUT   150000  /* Marker Delay TMO value in ms (150 sec) */
#define AMIQ_SELFTEST_TIMEOUT   10000   /* Selftest Timeout value in ms (10 sec) */
#define AMIQ_RAMTEST_TIMEOUT    300000  /* RAM Test max TMO value (5 min) */

/*= MACROS ==================================================================*/
#define CHECKERR(fCal) if (rsamiq_status = (fCal), rsamiq_status < VI_SUCCESS) \
							return rsamiq_status; else
#define CHECKERRCU(fCal) if (rsamiq_status = (fCal), rsamiq_status < VI_SUCCESS) \
							return rsamiq_initCleanUp (rmSession, instrSession, rsamiq_status); else


 
/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT COMMAND ARRAYS =====================================*/
/*****************************************************************************/
																			 
static ViString aDrive[] = {"A:","a:", VI_NULL};
static ViString PRBSArr[] = {"PRBS9","PRBS11","PRBS15","PRBS16","PRBS20",
							"PRBS21","PRBS23", VI_NULL};
static ViString BERTFileArr[] = {"SEL","COPY","DEL"};                  
static ViString BERTSignalArr[] = {"DATA","CLOC"};
static ViString BERTDataPolarArr[] = {"INV","NORM", VI_NULL};
static ViString BERTClockPolarArr[] = {"FALL","RIS", VI_NULL};
static ViString BERTMaskArr[] = {"OFF","HIGH","LOW"};
static ViString BERTRestartArr[] = {"EXT","INT", VI_NULL};
static ViString BERTCtrlArr[] = {"STOP","STAR"};
static ViString CalAmplOffsArr[] = {"AMPL", "OFFS", VI_NULL};
static ViString ClockFreqArr[] = {"SLOW","FAST", VI_NULL};
static ViString ChannelArr[] = {"I","Q"};
static ViString OutpClockArr[] = {"OFF","ON", VI_NULL};  
static ViString OutpExtClock[] = {"INT","EXTS", "EXTF", VI_NULL};  
static ViString OutpFilterArr[] = {"OFF","2.5MHZ","25MHZ","EXT", VI_NULL};  
static ViString OutpStateArr[] = {"OFF","FIX","VAR","INV", VI_NULL};
static ViString OutpImpArr[] = {"HIGH","R50", VI_NULL};
static ViString OutpTypeArr[] = {"UNB","BAL", VI_NULL};
static ViString RefOscArr[] = {"INT","EXT", VI_NULL};
static ViString CalibrationArr[] = {"ALL?","AMPL?","AMPL:I?","AMPL:Q?",
									"OFFS?","OFFS:I?","OFFS:Q?"};
static ViString MarkerOutArr[] = {"OFF","ON", VI_NULL};
static ViString TriggSourceArr[] = {"MAN", "BUS", "EXT", VI_NULL};
static ViString TriggModeArr[] = {"OFF", "GAT", "SING", "CONT", VI_NULL};
static ViString TriggSlopArr[] = {"NEG", "POS", VI_NULL};

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT STATUS/RANGE STRUCTURE  ============================*/
/*****************************************************************************/
/* rsamiq_stringValPair is used in the rsamiq_errorMessage function          */
/* rsamiq_statusDataRanges is used to track session dependent status & ranges*/
/*===========================================================================*/
typedef struct  rsamiq_stringValPair
{
   ViStatus stringVal;
   ViString stringName;
}  rsamiq_tStringValPair;
 
struct rsamiq_statusDataRanges {
	ViBoolean errorChecking;
	ViReal64  speed;
	ViReal64  flopySpeed;
	ViChar instrDriverRevision[256];
};

typedef struct rsamiq_statusDataRanges *rsamiq_instrRange;

/*****************************************************************************/
/*= UTILITY ROUTINE DECLARATIONS (Non-Exportable Functions) =================*/
/*****************************************************************************/
ViBoolean rsamiq_invalidViBooleanRange (ViBoolean val);
ViBoolean rsamiq_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max);
ViBoolean rsamiq_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max);
ViBoolean rsamiq_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max);
ViStatus rsamiq_initCleanUp (ViSession openRMSession, ViPSession openInstrSession,
				ViStatus currentStatus);
ViStatus rsamiq_readToFile (ViSession instrSession, ViString filename,
				ViUInt32 readBytes, ViPUInt32 retCount);
ViStatus rsamiq_writeFromFile (ViSession instrSession, ViString filename,
				ViUInt32 writeBytes, ViPUInt32 retCount);
ViStatus rsamiq_defaultInstrSetup (ViSession openInstrSession);

/*****************************************************************************/
/*------ INSERT INSTRUMENT-DEPENDENT UTILITY ROUTINE DECLARATIONS HERE ------*/
/*****************************************************************************/

ViStatus rsamiq_checkStatus (ViSession instrSession);
ViStatus rsamiq_waitOPC (ViSession instrSession, ViPString command,
				ViPString instrResp, ViInt32 timeout);
ViStatus rsamiq_SetInstrToVISABaudrate (ViSession instrSession);                
void rsamiq_Wait (ViReal64 timeToWait);

/*****************************************************************************/
/*====== USER-CALLABLE FUNCTIONS (Exportable Functions) =====================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Initialize                                                      */
/* Purpose:  This function opens the instrument, queries the instrument      */
/*           for its ID, and initializes the instrument to a known state.    */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_init (ViRsrc resourceName, ViBoolean IDQuery,
					ViBoolean resetDevice, ViPSession instrSession)
{
	ViStatus    rsamiq_status = VI_SUCCESS;
	ViSession   rmSession = 0;
	ViChar      rdBuffer[BUFFER_SIZE];
	ViUInt16    interface = VI_INTF_GPIB;

	/*- Check input parameter ranges ----------------------------------------*/
	if (rsamiq_invalidViBooleanRange (IDQuery))
		return VI_ERROR_PARAMETER2;
	if (rsamiq_invalidViBooleanRange (resetDevice))
		return VI_ERROR_PARAMETER3;

	/*- Open instrument session ---------------------------------------------*/
	CHECKERR (viOpenDefaultRM (&rmSession));
		
	if ((rsamiq_status = viOpen (rmSession, resourceName, VI_LOAD_CONFIG, VI_NULL, instrSession)) < 0) {
		viClose (rmSession);
		return rsamiq_status;
	}
	
	/* io session owned by driver now */
	CHECKERRCU( viGetAttribute (*instrSession, VI_ATTR_INTF_TYPE, &interface));

	/*- Configure VISA Formatted I/O ----------------------------------------*/
	CHECKERRCU( viSetAttribute (*instrSession, VI_ATTR_TMO_VALUE, AMIQ_DEFAULT_TIMEOUT));
	CHECKERRCU( viSetBuf (*instrSession, VI_READ_BUF|VI_WRITE_BUF, 4000));
	CHECKERRCU( viSetAttribute (*instrSession, VI_ATTR_WR_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));
	CHECKERRCU( viSetAttribute (*instrSession, VI_ATTR_RD_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));
	
	if (interface == VI_INTF_ASRL)  /* Serial Port */
		{
		CHECKERRCU( viSetAttribute (*instrSession, VI_ATTR_TERMCHAR, '\n'));
		CHECKERRCU( viSetAttribute (*instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE));
		CHECKERRCU( viSetAttribute (*instrSession, VI_ATTR_TERMCHAR_EN, VI_FALSE));  
		CHECKERRCU( viSetAttribute (*instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR));
		CHECKERRCU( viSetAttribute (*instrSession, VI_ATTR_ASRL_END_OUT, VI_ASRL_END_NONE));
		CHECKERRCU( viSetAttribute (*instrSession, VI_ATTR_ASRL_FLOW_CNTRL, VI_ASRL_FLOW_RTS_CTS));
		CHECKERRCU( rsamiq_SetInstrToVISABaudrate (*instrSession));
		}
	
	/*- Identification Query ------------------------------------------------*/
	if (IDQuery) {
		CHECKERRCU( viQueryf (*instrSession, "*IDN?\n", "%s", rdBuffer));
		if (strncmp (rdBuffer, "Rohde&Schwarz,AMIQ", 18))
			return rsamiq_initCleanUp (rmSession, instrSession, VI_ERROR_FAIL_ID_QUERY);
	}       

	/*- Reset instrument ----------------------------------------------------*/
	if (resetDevice)
		CHECKERRCU( rsamiq_reset (*instrSession));
	else  /*- Send Default Instrument Setup ---------------------------------*/
		CHECKERRCU( rsamiq_defaultInstrSetup (*instrSession));

	return rsamiq_status;
}

/*===========================================================================*/

/*===========================================================================*/
/* Function: Sine Wave                                                       */
/* Purpose:  This function generates an sine wave on the output of the AMIQ  */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_SineWave (ViSession instrSession)
{                               
	ViStatus rsamiq_status = VI_SUCCESS;
	ViReal64 data[1000];
	ViReal64 data2[1000];
	ViInt32 i;
	ViUInt32 bytesWritten;

	/* Generate sine wave */
	for(i=0;i<1000;i++)
	{
		data[i]=sin((ViReal64)i * (6.28/1000.0));
		data2[i]=-sin((ViReal64)i * (6.28/1000.0));/* Inv. phase */
	}   
			  
	/* Upload AMIQ RAM */
	CHECKERR (rsamiq_WaveformToRAM (instrSession, data, data2, 1000, &bytesWritten));

	/* Setup clock frequency (1MHz) */  
	CHECKERR (rsamiq_SetClkFreq (instrSession, 1000000.0, 0));
	
	/* Enable channel I */
	CHECKERR (rsamiq_SetOutState (instrSession, 0, 1));

	/* Enable channel Q */
	CHECKERR (rsamiq_SetOutState (instrSession, 1, 1));

	/* Go */
	CHECKERR (rsamiq_TriggImmediate (instrSession));

	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Easy BERT Config                                                */
/* Purpose:  his function configures all BERT parameters of the instrument.  */                        
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_EasyBERTConfig (ViSession instrSession,
										 ViInt32 PRBSLength, ViBoolean dataPolarity,
										 ViBoolean clockPolarity, ViInt32 masking,
										 ViUInt32 count,ViUInt32 errors,
										 ViInt32 restartSource)
{                               
	ViStatus rsamiq_status = VI_SUCCESS;
	
	if (rsamiq_invalidViInt32Range (PRBSLength, 0, 6))   
	   return VI_ERROR_PARAMETER2;
	if (rsamiq_invalidViBooleanRange (dataPolarity))
	   return VI_ERROR_PARAMETER3;
	if (rsamiq_invalidViBooleanRange (clockPolarity))
	   return VI_ERROR_PARAMETER4;
	if (rsamiq_invalidViInt32Range (masking, 0, 2))  
	   return VI_ERROR_PARAMETER5;
	if (rsamiq_invalidViUInt32Range (count, 1, ULONG_MAX))  
	   return VI_ERROR_PARAMETER6;
	if (rsamiq_invalidViUInt32Range (errors, 1, ULONG_MAX))     
	   return VI_ERROR_PARAMETER7;
	if (rsamiq_invalidViInt32Range (restartSource, 0, 1))    
	   return VI_ERROR_PARAMETER8;

	CHECKERR (viPrintf (instrSession, ":BERT:SET:TYPE %s;:BERT:SET:CLOC %s;"
		":BERT:SET:DATA %s;:BERT:SET:MASK %s;:BERT:SET:MCO %lu;:BERT:SET:MERR %lu;"
		":BERT:SET:REST %s\n", PRBSArr[PRBSLength], BERTClockPolarArr[clockPolarity],
		BERTDataPolarArr[dataPolarity], BERTMaskArr[masking], count, errors,
		BERTRestartArr[restartSource]));   
	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: BERT File                                                       */
/* Purpose:  This function manipulates the BERT file. The BERT file is a     */
/*           file containing the configuration information for the           */
/*           (reprogrammable) hardware.                                      */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_BERTFile (ViSession instrSession, ViInt32 mode,
								   ViString file)
{                               
	ViStatus rsamiq_status = VI_SUCCESS;

	if (rsamiq_invalidViInt32Range (mode, 0, 2))
	   return VI_ERROR_PARAMETER2;

	CHECKERR (viPrintf (instrSession, ":BERT:%s \"%s\"\n", BERTFileArr[mode], file));
	CHECKERR (rsamiq_checkStatus (instrSession));

	return rsamiq_status;
}

/*===========================================================================*/
/* Function: BERT File Query                                                 */
/* Purpose:  This function queries the BERT file.                            */
/*           BERT file containing the configuration information for the      */
/*           (reprogrammable) hardware.                                      */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_BERTFileQuery (ViSession instrSession, ViChar _VI_FAR file[])
{                               
	ViStatus rsamiq_status = VI_SUCCESS;

	if(!file) return VI_ERROR_PARAMETER2;

	CHECKERR (viQueryf (instrSession, ":BERT:SELECT?\n","%s",file));
	CHECKERR (rsamiq_checkStatus (instrSession));

	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Set PRBS Type                                                   */
/* Purpose:  This function sets the PRBS type.                               */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_SetPRBSType (ViSession instrSession, ViInt32 type)
{                               
	ViStatus rsamiq_status = VI_SUCCESS;
	
	if (rsamiq_invalidViInt32Range (type, 0, 6))     
	   return VI_ERROR_PARAMETER2;
   
	CHECKERR (viPrintf (instrSession, ":BERT:SET:TYPE %s\n", PRBSArr[type]));
	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Get PRBS Type                                                   */
/* Purpose:  This function gets the PRBS type.                               */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_GetPRBSType (ViSession instrSession, ViPInt32 type)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;
	ViChar      buffer[BUFFER_SIZE];
	ViInt32     i;

	if (!type) return VI_ERROR_PARAMETER2;

	CHECKERR (viQueryf (instrSession, ":BERT:SET:TYPE?\n", "%s", buffer));
	for (i = 0; PRBSArr[i]; i++)
		if (!strcmp(buffer, PRBSArr[i])) break;
		
	if (!PRBSArr[i])
		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	*type = i;

	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Set BERT Polarity                                               */
/* Purpose:  This function sets the polarity of the external data and clock  */
/*           inputs.                                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_SetBERTPol (ViSession instrSession, ViBoolean signal,
	ViBoolean polarity)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;
	ViChar      buffer[BUFFER_SIZE];
	ViInt32     tmpCnt;
	
	if (rsamiq_invalidViBooleanRange (signal))   
	   return VI_ERROR_PARAMETER2;
	if (rsamiq_invalidViBooleanRange (polarity))     
	   return VI_ERROR_PARAMETER3;
   
	tmpCnt = sprintf(buffer, ":BERT:SET:%s", BERTSignalArr[signal]);
	if (signal)
		sprintf(buffer + tmpCnt, " %s\n", BERTClockPolarArr[polarity]);
	else
		sprintf(buffer + tmpCnt, " %s\n", BERTDataPolarArr[polarity]);

	CHECKERR (viWrite (instrSession, (ViBuf)buffer, (ViUInt32) strlen(buffer), VI_NULL));
	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Get BERT Polarity                                               */
/* Purpose:  This function sets the polarity of the external data and clock  */
/*           inputs.                                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_GetBERTPol (ViSession instrSession, ViBoolean signal,
	ViPBoolean polarity)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;
	ViChar      buffer[BUFFER_SIZE];
	ViUInt16    i;

	if (rsamiq_invalidViBooleanRange (signal))   
	   return VI_ERROR_PARAMETER2;
   
	if (!polarity)
		return VI_ERROR_PARAMETER3;

	if (polarity) /* CLOCK */ {
		CHECKERR (viPrintf (instrSession, ":BERT:SET:%s?\n", BERTSignalArr[signal]));
		CHECKERR (viScanf (instrSession, "%s", buffer));
		
		if (signal) {
			for (i = 0; BERTClockPolarArr[i]; i++)
				if (!strcmp(buffer, BERTClockPolarArr[i])) break;
			if (!BERTClockPolarArr[i])
				return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		}
		else {
			for (i = 0; BERTDataPolarArr[i]; i++)
				if (!strcmp(buffer, BERTDataPolarArr[i])) break;
			if (!BERTClockPolarArr[i])
				return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		}       
		
		*polarity = i;
	}
	
	CHECKERR (rsamiq_checkStatus (instrSession));
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Set BERT Masking                                                */
/* Purpose:  This function sets the BERT masking. AMIQ is provided with an   */
/*           input (Data Enable) which permits the BER measurement to be     */
/*           temporarily disabled so that data bursts or data interrupted by */
/*           other data can be processed. The data enable signal is supplied */
/*           to pin 8 of the BER connector.                                  */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_SetBERTMask (ViSession instrSession, ViInt32 mode)
{                               
	ViStatus rsamiq_status = VI_SUCCESS;
	
	if (rsamiq_invalidViInt32Range (mode, 0, 2))     
	   return VI_ERROR_PARAMETER2;
   
	CHECKERR (viPrintf (instrSession, ":BERT:SET:MASK %s\n", BERTMaskArr[mode]));   
	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Get BERT Masking                                                */
/* Purpose:  This function sets the BERT masking. AMIQ is provided with an   */
/*           input (Data Enable) which permits the BER measurement to be     */
/*           temporarily disabled so that data bursts or data interrupted by */
/*           other data can be processed. The data enable signal is supplied */
/*           to pin 8 of the BER connector.                                  */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_GetBERTMask (ViSession instrSession, ViPInt32 mode)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;
	ViChar      buffer[BUFFER_SIZE];
	ViInt32     i;

	if (!mode) return VI_ERROR_PARAMETER2;

	CHECKERR (viQueryf (instrSession, ":BERT:SET:MASK?\n", "%s", buffer));
	for (i = 0; BERTMaskArr[i]; i++)
		if (!strcmp(buffer, BERTMaskArr[i])) break;
	
	if (!BERTMaskArr[i])
		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	*mode = i;

	CHECKERR (rsamiq_checkStatus (instrSession));
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Set BERT Complete Condition                                     */
/* Purpose:  This function sets the BERT stop condition. When this condition */
/*           is met the BERT measurement is terminated.                      */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_SetBERTCond (ViSession instrSession, ViUInt32 count,
	ViUInt32 errors)
{
	ViStatus rsamiq_status = VI_SUCCESS;
	
	if (rsamiq_invalidViUInt32Range (count, 1, ULONG_MAX))
	   return VI_ERROR_PARAMETER2;
	if (rsamiq_invalidViUInt32Range (errors, 1, ULONG_MAX))
	   return VI_ERROR_PARAMETER3;
   
	CHECKERR (viPrintf (instrSession, ":BERT:SET:MCO %lu;:BERT:SET:MERR %lu\n",
						count, errors));    
	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Get BERT Complete Condition                                     */
/* Purpose:  This function gets the BERT stop condition. When this condition */
/*           is met the BERT measurement is terminated.                      */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_GetBERTCond (ViSession instrSession, ViPUInt32 count,
	ViPUInt32 errors)
{
	ViStatus    rsamiq_status = VI_SUCCESS;

	if(!count) return VI_ERROR_PARAMETER2;
	if(!errors) return VI_ERROR_PARAMETER3;

	CHECKERR (viQueryf (instrSession, ":BERT:SET:MCO?\n", "%lu",count));
	CHECKERR (viQueryf (instrSession, ":BERT:SET:MERR?\n", "%lu",errors));

	CHECKERR (rsamiq_checkStatus (instrSession));

	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Set BERT Restart                                                */
/* Purpose:  This function sets the BERT restart signal source.              */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_SetBERTRestart (ViSession instrSession, ViInt32 source)
{
	ViStatus rsamiq_status = VI_SUCCESS;

	if (rsamiq_invalidViInt32Range (source, 0, 1))   
	   return VI_ERROR_PARAMETER2;

	CHECKERR (viPrintf (instrSession, ":BERT:SET:REST %s\n", BERTRestartArr[source]));    
	CHECKERR (rsamiq_checkStatus (instrSession));

	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Get BERT Restart                                                */
/* Purpose:  This function gets the BERT restart signal source.              */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_GetBERTRestart (ViSession instrSession, ViPInt32 source)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;
	ViChar      buffer[BUFFER_SIZE];
	ViInt32     i;

	if (!source) return VI_ERROR_PARAMETER2;

	CHECKERR (viQueryf (instrSession, ":BERT:SET:REST?\n", "%s", buffer));
	for (i = 0; BERTRestartArr[i]; i++)
		if (!strcmp(buffer, BERTRestartArr[i])) break;
		
	if (!BERTRestartArr[i])
		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	*source = i;

	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Get BERT Result                                                 */
/* Purpose:  This function gets result of the last BER measurement..         */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_GetBERTResult (ViSession instrSession,
							   ViPInt32 dataBits,
							   ViPInt32 errorBits,
							   ViPReal64 errorRate,
							   ViPBoolean termStatus,
							   ViPBoolean clockLineStatus,
							   ViPBoolean dataLineStatus,
							   ViPBoolean syncStatus)
{
	ViStatus    rsamiq_status = VI_SUCCESS;

	if (!dataBits) return VI_ERROR_PARAMETER2;
	if (!errorBits) return VI_ERROR_PARAMETER3;
	if (!errorRate) return VI_ERROR_PARAMETER4;
	if (!termStatus) return VI_ERROR_PARAMETER5;
	if (!clockLineStatus) return VI_ERROR_PARAMETER6;
	if (!dataLineStatus) return VI_ERROR_PARAMETER7;
	if (!syncStatus) return VI_ERROR_PARAMETER8;

	CHECKERR (viQueryf (instrSession, ":BERT:RES?\n","%d,%d,%lf,%hu,%hu,%hu,%hu",
						dataBits, errorBits, errorRate, termStatus,
						clockLineStatus, dataLineStatus, syncStatus));    
	CHECKERR (rsamiq_checkStatus (instrSession));

	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Calibration Values                                              */
/* Purpose:  This function returns an array of calibration values stored in  */
/*           the instruments EEPROM or on the hard disk. The calibration     */
/*           values can not be set on the instrument from the  controller.   */
/*           The can only be determined by performing the instrument         */
/*           calibration.                                                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_CalVals (ViSession instrSession, ViInt32 mode,
	ViInt32 _VI_FAR values[])
{
	ViStatus    rsamiq_status = VI_SUCCESS;
	/* 1000 bytes is enough for 58 calibration values */
	ViByte      tempBuffer[1000];
	ViPByte     p2Buffer = tempBuffer;
	ViUInt32    bytesRead;
	ViInt32     idx = 0;
	ViInt32     i;

	if (rsamiq_invalidViInt32Range (mode, 0, 1))
	   return VI_ERROR_PARAMETER2;

	if (!values) return VI_ERROR_PARAMETER3;

	CHECKERR (viPrintf (instrSession, ":CAL:%s:VAL?\n",CalAmplOffsArr[mode]));
	CHECKERR (viRead (instrSession, tempBuffer,1000,&bytesRead));

	for(i = 0; i < (ViInt32)bytesRead; i++){
		if(tempBuffer[i]==',' || tempBuffer[i]=='\n'){
			tempBuffer[i]=0;
			if (1==sscanf((char *)p2Buffer,"%d",&values[idx])) idx++;
			p2Buffer += strlen((char *)p2Buffer)+1;
		}
	}

	if(mode){
	   if(idx!=58) return VI_ERROR_INSTR_INTERPRETING_RESPONSE; 
	}else{
		if(idx!=18) return VI_ERROR_INSTR_INTERPRETING_RESPONSE;    
	}

	CHECKERR (rsamiq_checkStatus (instrSession));

	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Set ADC Cal Factor                                              */
/* Purpose:  This function sets the calibration factor for the diagnostic A/D*/
/*           converter with the aid of which the measurement errors of the   */
/*           converter can be compensated for. Each measured value is        */ 
/*           multiplied by this calibration factor before it is output.      */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_SetADCCalFact (ViSession instrSession, ViReal64 factor)
{
	ViStatus    rsamiq_status = VI_SUCCESS;

	if (rsamiq_invalidViReal64Range (factor, 0.9, 1.1))  
	   return VI_ERROR_PARAMETER2;

	CHECKERR (viPrintf (instrSession, ":CAL:DIAG %lf\n", factor));
	CHECKERR (rsamiq_checkStatus (instrSession));

	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Get ADC Cal Factor                                              */
/* Purpose:  This function gets the calibration factor for the diagnostic A/D*/
/*           converter with the aid of which the measurement errors of the   */
/*           converter can be compensated for. Each measured value is        */ 
/*           multiplied by this calibration factor before it is output.      */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_GetADCCalFact (ViSession instrSession, ViPReal64 factor)
{
	ViStatus    rsamiq_status = VI_SUCCESS;

	if (!factor) return VI_ERROR_PARAMETER2;
	
	CHECKERR (viQueryf (instrSession, ":CAL:DIAG?\n", "%lf", factor));

	CHECKERR (rsamiq_checkStatus (instrSession));
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Set Ref Osc Calibration                                         */
/* Purpose:  This function sets the tuning voltage for the internal 10 MHz   */
/*           reference oscillator. With the aid of an external counter, the  */
/*           internal reference oscillator can thus be accurately adjusted   */
/*           to 10 MHz.                                                      */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_SetRefOscCal (ViSession instrSession, ViInt32 factor)
{
	ViStatus    rsamiq_status = VI_SUCCESS;

	if (rsamiq_invalidViInt32Range (factor, 0, 4095))
	   return VI_ERROR_PARAMETER2;

	CHECKERR (viPrintf (instrSession, ":CAL:ROSC %ld\n", factor));
	CHECKERR (rsamiq_checkStatus (instrSession));

	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Get Ref Osc Calibration                                         */
/* Purpose:  This function gets the tuning voltage for the internal 10 MHz   */
/*           reference oscillator. With the aid of an external counter, the  */
/*           internal reference oscillator can thus be accurately adjusted   */
/*           to 10 MHz.                                                      */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_GetRefOscCal (ViSession instrSession, ViPInt32 factor)
{
	ViStatus    rsamiq_status = VI_SUCCESS;

	if (!factor) return VI_ERROR_PARAMETER2;

	CHECKERR (viQueryf (instrSession, ":CAL:ROSC?\n", "%ld", factor));
	CHECKERR (rsamiq_checkStatus (instrSession));

	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Set Markers                                                     */
/* Purpose:  This function sets the current marker list. Two binary marker   */
/*           outputs are available for each output.                          */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_SetMarkers (ViSession instrSession,
									 ViInt32 marker, ViInt32 _VI_FAR startIndex[],
									 ViInt32 _VI_FAR stopIndex[],
									 ViBoolean _VI_FAR markerState[],
									  ViInt32 numberOfPoints)
{
	ViStatus rsamiq_status = VI_SUCCESS;
	ViPChar tempBuffer, p2Buffer;
	ViInt32 i;

	if (rsamiq_invalidViInt32Range (marker, 1, 4))
		return VI_ERROR_PARAMETER2;

	if (!startIndex)
		return VI_ERROR_PARAMETER3;

	if (!stopIndex)
		return VI_ERROR_PARAMETER4;

	if (!markerState)
		return VI_ERROR_PARAMETER5;

	if(numberOfPoints){

		/* Allocate temp buffer *//* 18 bytes per marker (worst case value) */
		if((p2Buffer=tempBuffer=(ViPChar)malloc(numberOfPoints*18+8)) == VI_NULL) 
			return VI_ERROR_ALLOC;

		/* Place cmd to buffer */
		sprintf(p2Buffer,"MARK%d \"",marker);

		/* Update p2Buffer pointer */
		p2Buffer+=strlen(p2Buffer);

		/* Place markes to buffer */
		for(i=0;i<numberOfPoints;i++){

			sprintf(p2Buffer,"%d-%d:%hu;",startIndex[i],stopIndex[i],markerState[i]);

			/* Update p2Buffer pointer */
			p2Buffer+=strlen(p2Buffer); 
		}

		/* Add '"' at the end of buffer */
		*--p2Buffer='"';

		if (rsamiq_status = (viPrintf (instrSession, "%s\n",tempBuffer)), rsamiq_status < VI_SUCCESS){

			/* Free temp buffer */
			free(tempBuffer);

			return rsamiq_status; 

		}

		/* Free temp buffer */
		free(tempBuffer);
	}

	CHECKERR (rsamiq_checkStatus (instrSession));

	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Get Markers                                                     */
/* Purpose:  This function gets the current marker list from the instrument. */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_GetMarkers (ViSession instrSession,ViInt32 marker,
									 ViInt32 _VI_FAR startIndex[],
									 ViInt32 _VI_FAR stopIndex[],
									 ViBoolean _VI_FAR markerState[],
									  ViPInt32 numberofPoints)
{
	ViChar      buffer[BUFFER_SIZE];
	ViUInt32    bufferSize;
	size_t      wrapSize = 0;
	ViInt32     i,dataIdx=0;
	ViPChar     p2Buffer;
	ViBoolean   errorFlag=VI_FALSE,lastTurn=VI_FALSE;

	ViStatus rsamiq_status = VI_SUCCESS;

	if (rsamiq_invalidViInt32Range (marker, 1, 4))
	   return VI_ERROR_PARAMETER2;
	if (!startIndex) return VI_ERROR_PARAMETER3;
	if (!stopIndex) return VI_ERROR_PARAMETER4; 
	if (!markerState) return VI_ERROR_PARAMETER5;
	if (!numberofPoints) return VI_ERROR_PARAMETER6;

	/* Send Cmd */
	CHECKERR (viPrintf (instrSession, "MARK%d?\n",marker));

	/* Read  and parse response */
	while(VI_TRUE){

		/* End of data stream has reached */
		if(lastTurn) break;

		CHECKERR (viRead (instrSession, (ViPBuf)buffer+wrapSize,BUFFER_SIZE-wrapSize,
						&bufferSize));

		bufferSize+=wrapSize;
		p2Buffer=(ViChar *)buffer;

		/* Parse buffer */
		for(i=0; i < (ViInt32)bufferSize; i++){
			if( buffer[i]==';'|| (lastTurn=(ViBoolean)(buffer[i]=='\n'))== 1) {
				buffer[i]=0;
				if(!errorFlag){
					if(sscanf(p2Buffer,"%d-%d:%hu",&startIndex[dataIdx],
												  &stopIndex[dataIdx],
												  &markerState[dataIdx])!=3) errorFlag=1;
					dataIdx++;
				}
				
				p2Buffer=(ViChar *)buffer+i+1;
				if((i+1) == (ViInt32)bufferSize || lastTurn ) break;
			}
		}
			
		if (lastTurn){
			wrapSize = 0;
			break;
		}

		/* Wrap buffer as needed */
		if(i == (ViInt32)bufferSize){
			wrapSize=buffer+bufferSize-p2Buffer;
			memcpy (buffer, p2Buffer, wrapSize);
		}else{
			wrapSize=0;
		}
	}

	if(errorFlag) return VI_ERROR_INSTR_INTERPRETING_RESPONSE;  

	*numberofPoints=dataIdx;

	CHECKERR (rsamiq_checkStatus (instrSession));

	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Set Marker Ouput                                                */
/* Purpose:  This function sets whether the specified marker output is on    */
/*           or off.                                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_SetMkrOut (ViSession instrSession, ViInt32 marker,
	ViBoolean state)
{
	ViStatus    rsamiq_status = VI_SUCCESS;

	if (rsamiq_invalidViInt32Range (marker, 1, 4))
	   return VI_ERROR_PARAMETER2;
	if (rsamiq_invalidViBooleanRange (state))
	   return VI_ERROR_PARAMETER3;

	CHECKERR (viPrintf (instrSession, ":OUTP:MARK%d %s\n", marker, MarkerOutArr[state]));
	CHECKERR (rsamiq_checkStatus (instrSession));

	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Get Marker Ouput                                                */
/* Purpose:  This function gets whether the specified marker output is on    */
/*           or off.                                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_GetMkrOut (ViSession instrSession, ViInt32 marker,
	ViPBoolean state)
{
	ViStatus    rsamiq_status = VI_SUCCESS;
	ViChar      buffer[BUFFER_SIZE];
	ViUInt16    i;

	if (rsamiq_invalidViInt32Range (marker, 1, 4))
	   return VI_ERROR_PARAMETER2;

	if (!state) return VI_ERROR_PARAMETER3;

	CHECKERR (viQueryf (instrSession, ":OUTP:MARK%d?\n", "%s", marker, buffer));
	for (i = 0; MarkerOutArr[i]; i++)
		if (!strcmp(buffer, MarkerOutArr[i])) break;
	
	if (!MarkerOutArr[i])
		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	*state = i;

	CHECKERR (rsamiq_checkStatus (instrSession));

	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Set Marker Delay                                                */
/* Purpose:  This function sets the marker delay.                            */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_SetMkrDly (ViSession instrSession, ViInt32 marker,
	ViInt32 delay)
{
	ViStatus    rsamiq_status = VI_SUCCESS;
	ViInt32     oldTimeout;

	if (rsamiq_invalidViInt32Range (marker, 1, 4))   
	   return VI_ERROR_PARAMETER2;

	CHECKERR (viGetAttribute (instrSession, VI_ATTR_TMO_VALUE, &oldTimeout));
	CHECKERR (viSetAttribute (instrSession, VI_ATTR_TMO_VALUE,
		MAX_AMIQ_MARKER_SHIFT_TIMEOUT));

	CHECKERR (viPrintf (instrSession, ":OUTP:MARK%d:DEL %d\n", marker, delay));

	CHECKERR (viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, oldTimeout));

	CHECKERR (rsamiq_checkStatus (instrSession));

	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Get Marker Delay                                                */
/* Purpose:  This function gets the marker delay.                            */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_GetMkrDly (ViSession instrSession, ViInt32 marker,
	ViPInt32 delay)
{
	ViStatus rsamiq_status = VI_SUCCESS;

	if (rsamiq_invalidViInt32Range (marker, 1, 4))   
	   return VI_ERROR_PARAMETER2;
	if(!delay) return VI_ERROR_PARAMETER3;

	CHECKERR (viQueryf (instrSession, ":OUTP:MARK%d:DEL?\n", "%d", marker, delay));
	CHECKERR (rsamiq_checkStatus (instrSession));

	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Mem Dir                                                         */
/* Purpose:  This function lists the contens of current directory in the     */
/*           instrument internal memory.                                     */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_memDir (ViSession instrSession, ViInt32 mode,
								 ViChar _VI_FAR fileList[], ViInt32 bufferSize,
								 ViPInt32 numberOfItems)
{
	ViStatus rsamiq_status = VI_SUCCESS;
	ViPChar p2Buffer = fileList;
	ViInt32 i;
	ViBoolean firstTurn = 1;
	ViBoolean nextValid;
	ViString list[] = {"CAT", "DCAT", "SCAT"};

	if (rsamiq_invalidViInt32Range (mode, 0, 2))
	   return VI_ERROR_PARAMETER2;
	if (!numberOfItems) return VI_ERROR_PARAMETER4;

	(*numberOfItems) = 0;

	/*  list buffer is empty */
	if (!fileList) {

		CHECKERR (viQueryf (instrSession, ":MMEM:%s:LENG?\n",
			"%d", list[mode], numberOfItems));  

		CHECKERR (rsamiq_checkStatus (instrSession));   

		return rsamiq_status;
	}

	/* Send command */
	CHECKERR (viPrintf (instrSession, ":MMEM:%s?\n", list[mode]));

	/* Read response */
	CHECKERR (viRead (instrSession, (ViPBuf)fileList, bufferSize, (ViPUInt32)&bufferSize)); 

	if(bufferSize){

		/* Parse response buffer */
		for(i=0;i<bufferSize;i++){
		
			if(fileList[i]=='\n'){
				(*numberOfItems)++;
				break;
			}

			if (!mode && !firstTurn && (i+1)!=bufferSize)
				nextValid = (ViBoolean)(fileList[i+1] == '"');
				else nextValid = 0;

			if(fileList[i] == ',' && (firstTurn || mode || nextValid )) {
				firstTurn = 0;
				fileList[i]='\n';
				p2Buffer=&fileList[i];
				(*numberOfItems)++;
			}


		}

		if(i==bufferSize) *p2Buffer=0;
			else fileList[i]=0;
	}

	/* Remove "used bytes" and "free bytes" items from count */
	if(!mode) (*numberOfItems)-=2;

	/* Coerce numberOfItems variable */
	if((*numberOfItems)<0) (*numberOfItems) = 0;

	CHECKERR (rsamiq_checkStatus (instrSession));

	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Mem Copy                                                        */
/* Purpose:  This function copies source file into destination.              */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_memCopy (ViSession instrSession, ViString source,
								  ViString destination)
{
	ViStatus    rsamiq_status = VI_SUCCESS;

	if(!source) return VI_ERROR_PARAMETER2;
	if(!destination) return VI_ERROR_PARAMETER3;

	CHECKERR (viPrintf (instrSession, ":MMEM:COPY \"%s\",\"%s\"\n", source, destination));
	CHECKERR (rsamiq_checkStatus (instrSession));

	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Mem Delete                                                      */
/* Purpose:  This function deletes the specified file.                       */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_memDel (ViSession instrSession, ViString name)
{
	ViStatus    rsamiq_status = VI_SUCCESS;

	if(!name) return VI_ERROR_PARAMETER2;

	CHECKERR (viPrintf (instrSession, ":MMEM:DEL \"%s\"\n", name));
	CHECKERR (rsamiq_checkStatus (instrSession));

	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Mem Change Directory                                            */
/* Purpose:  This function changes the default directory of the instrument's */
/*           file system.                                                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_memSetCd (ViSession instrSession, ViString directory)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;
	
	if(!directory) return VI_ERROR_PARAMETER2;  
	
	CHECKERR (viPrintf (instrSession, ":MMEM:CDIR \"%s\"\n", directory));    
	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Mem Get Directory                                               */
/* Purpose:  This function queries the current directory of the instrument's */
/*           fiel system.                                                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_memGetCd (ViSession instrSession,
								   ViChar _VI_FAR directory[])
{
	ViStatus    rsamiq_status = VI_SUCCESS;

	if(!directory) return VI_ERROR_PARAMETER2;  

	CHECKERR (viQueryf (instrSession, ":MMEM:CDIR?\n","%s", directory));    
	CHECKERR (rsamiq_checkStatus (instrSession));

	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Mem Make Directory                                              */
/* Purpose:  This function creates specified directory on the instrument's   */
/*           internal file system.                                           */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_memMd (ViSession instrSession, ViString directory)
{
	ViStatus    rsamiq_status = VI_SUCCESS;

	if(!directory) return VI_ERROR_PARAMETER2;  

	CHECKERR (viPrintf (instrSession, ":MMEM:MD \"%s\"\n", directory));    
	CHECKERR (rsamiq_checkStatus (instrSession));

	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Mem Remove Directory                                            */
/* Purpose:  This function removes the specified directory from the          */
/*           instruments file system.                                        */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_memRd (ViSession instrSession, ViString directory)
{
	ViStatus    rsamiq_status = VI_SUCCESS;

	if(!directory) return VI_ERROR_PARAMETER2;  

	CHECKERR (viPrintf (instrSession, ":MMEM:RD \"%s\"\n", directory));    
	CHECKERR (rsamiq_checkStatus (instrSession));

	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Mem Set Drive                                                   */
/* Purpose:  This function sets the current drive of the instrument's file   */
/*           system.                                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_memSetDrv (ViSession instrSession, ViString drive)
{
	ViStatus    rsamiq_status = VI_SUCCESS;

	if(!drive) return VI_ERROR_PARAMETER2;  

	CHECKERR (viPrintf (instrSession, ":MMEM:MSIS \"%s\"\n", drive));    
	CHECKERR (rsamiq_checkStatus (instrSession));

	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Mem Get Drive                                                   */
/* Purpose:  This function queries the current drive of the instrument's file*/
/*           system.                                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_memGetDrv (ViSession instrSession, ViChar _VI_FAR drive[])
{
	ViStatus    rsamiq_status = VI_SUCCESS;

	if(!drive) return VI_ERROR_PARAMETER2;  

	CHECKERR (viQueryf (instrSession, ":MMEM:MSIS?\n","%s", drive));    
	CHECKERR (rsamiq_checkStatus (instrSession));

	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Mem Load                                                        */
/* Purpose:  This function loads the waveform file into the instrument's RAM.*/
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_memLoad (ViSession instrSession, ViString name)
{
	ViStatus    rsamiq_status = VI_SUCCESS;
	ViReal64    speed = 400.0;
	ViInt32     newTimeout = 10000,
				oldTimeout, i,
				file_size = 16000000L;
	ViChar      buffer[10];
	rsamiq_instrRange instrPtr;

	if(!name) return VI_ERROR_PARAMETER2;

	CHECKERR( viQueryf (instrSession, ":MMEM:DATA:LENG? \"%s\"\n", "%ld", name,
		&file_size));

	CHECKERR( viQueryf (instrSession, ":MMEM:CDIR?\n", "%s", buffer));
	CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
	speed = instrPtr -> speed;

	for (i = 0; aDrive[i]; i++)
		if (strstr(buffer, aDrive[i]) || strstr(name, aDrive[i]))
			{
			ViReal64 flopySpeed;
			flopySpeed = instrPtr -> flopySpeed;
			newTimeout = (ViInt32)MAX_AMIQ_FLOPPY_FILE_LOAD_TIMEOUT;
			break;
			}
		else
			newTimeout = (ViInt32)MAX_AMIQ_HD_FILE_LOAD_TIMEOUT;

	CHECKERR (rsamiq_checkStatus (instrSession));

	CHECKERR (viGetAttribute (instrSession, VI_ATTR_TMO_VALUE, &oldTimeout));
	CHECKERR (viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, newTimeout));

	CHECKERR (viPrintf (instrSession, ":MMEM:LOAD RAM, \"%s\"\n", name));    

	CHECKERR (viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, oldTimeout));

	CHECKERR (rsamiq_checkStatus (instrSession));

	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Mem Name                                                        */
/* Purpose:  This function gets the name of the waveform file in the output  */
/*           buffer.                                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_memName (ViSession instrSession, ViChar _VI_FAR name[])
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;

	if(!name) return VI_ERROR_PARAMETER2;

	CHECKERR (viQueryf (instrSession, ":MEM:NAME?\n", "%s", name));
	CHECKERR (rsamiq_checkStatus (instrSession));    

	return rsamiq_status;
}


/*===========================================================================*/
/* Function: Set Clock Frequency                                             */
/* Purpose:  This function sets the instrument clock frequency and mode.     */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_SetClkFreq (ViSession instrSession, ViReal64 frequency,
	ViBoolean mode)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;
	
	if (rsamiq_invalidViReal64Range (frequency, 10, 105e+6))
	   return VI_ERROR_PARAMETER2;
	if (rsamiq_invalidViBooleanRange (mode))     
	   return VI_ERROR_PARAMETER3;
		
	CHECKERR (viPrintf (instrSession, ":SOUR:CLOC %lf, %s\n", frequency, 
		ClockFreqArr[mode]));    

	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Get Clock Frequency                                             */
/* Purpose:  This function gets the instrument clock frequency and mode.     */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_GetClkFreq (ViSession instrSession, ViPReal64 frequency)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;

	if (!frequency) return VI_ERROR_PARAMETER2;
	
	CHECKERR (viQueryf (instrSession, ":SOUR:CLOC?\n","%lf",frequency));    
	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Set Corrections                                                 */
/* Purpose:  This function sets gain and offset corrections.                 */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_SetCorr (ViSession instrSession, ViInt32 channel,
	ViReal64 gain, ViReal64 offset)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;
	
	if (rsamiq_invalidViInt32Range (channel, 0 ,1))  
	   return VI_ERROR_PARAMETER2;
	if (rsamiq_invalidViReal64Range (gain, -1.0, 1.0))   
	   return VI_ERROR_PARAMETER3;    
	if (rsamiq_invalidViReal64Range (offset, -1.0, 1.0))     
	   return VI_ERROR_PARAMETER4;    
	 
	CHECKERR (viPrintf (instrSession, ":CORR:GAIN:%s:FIX %lf;:CORR:OFFS:%s:FIX %lf\n", 
		ChannelArr[channel], gain, ChannelArr[channel], offset));    
		
	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Get Corrections                                                 */
/* Purpose:  This function gets gain and offset corrections.                 */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_GetCorr (ViSession instrSession, ViInt32 channel,
	ViPReal64 gain, ViPReal64 offset)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;
	
	if (rsamiq_invalidViInt32Range (channel, 0 ,1))  
	   return VI_ERROR_PARAMETER2;
	if (!gain)
	   return VI_ERROR_PARAMETER3;
	if (!offset)
	   return VI_ERROR_PARAMETER4;
	
	CHECKERR (viQueryf (instrSession, ":CORR:GAIN:%s:FIX?\n", "%lf",
						ChannelArr[channel], gain));
	CHECKERR (viQueryf (instrSession, ":CORR:OFFS:%s:FIX?\n", "%lf",
						ChannelArr[channel], offset));

	CHECKERR (rsamiq_checkStatus (instrSession));

	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Set Skew                                                        */
/* Purpose:  This function sets the delay between I and Q channels.          */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_SetSkew (ViSession instrSession, ViReal64 skew)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;
	
	if (rsamiq_invalidViReal64Range (skew, -1.0, 1.0))   
	   return VI_ERROR_PARAMETER2;    
	 
	CHECKERR (viPrintf (instrSession, ":CORR:SKEW %lf\n", skew)); 
	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Get Skew                                                        */
/* Purpose:  This function gets the delay between I and Q channels.          */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_GetSkew (ViSession instrSession, ViPReal64 skew)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;
	
	if (!skew) VI_ERROR_PARAMETER2;    

	CHECKERR (viQueryf (instrSession, ":CORR:SKEW?\n", "%lf", skew));
	
	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Set Clock Output                                                */
/* Purpose:  This function sets whether the sampling clock of the waveform   */
/*           D/A converter is connected to the clock output or not.          */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_SetClkOut (ViSession instrSession, ViBoolean state)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;
	
	if (rsamiq_invalidViBooleanRange (state))    
	   return VI_ERROR_PARAMETER2;    
	 
	CHECKERR (viPrintf (instrSession, ":OUTP:CLOC %s\n", OutpClockArr[state])); 
	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Get Clock Output                                                */
/* Purpose:  This function returns whether the sampling clock of the waveform*/
/*           D/A converter is connected to the clock output or not.          */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_GetClkOut (ViSession instrSession, ViPBoolean state)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;
	ViChar      buffer[BUFFER_SIZE];
	ViUInt16        i;
	
	if (!state) return VI_ERROR_PARAMETER2;    
	
	CHECKERR (viQueryf (instrSession, ":OUTP:CLOC?\n", "%s", buffer));
	
	for (i = 0; OutpClockArr[i]; i++)
		if (!strcmp(buffer, OutpClockArr[i])) break;
	if (!OutpClockArr[i])
		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
			
	*state = i;

	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Set Ouput Filter                                                */
/* Purpose:  This function sets the reconstruction filters to be cut into    */
/*           the output signal path.                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_SetOutFlt (ViSession instrSession, ViInt32 channel,
	ViInt32 filter)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;
	
	if (rsamiq_invalidViInt32Range (channel, 0, 2))  
	   return VI_ERROR_PARAMETER2;
	if (rsamiq_invalidViInt32Range (filter, 0, 3))   
	   return VI_ERROR_PARAMETER3;    
		
	if (channel<2)
	   CHECKERR (viPrintf (instrSession, ":OUTP:%s:FILT %s\n", ChannelArr[channel],
						   OutpFilterArr[filter])); 
	else    
	   CHECKERR (viPrintf (instrSession, ":OUTP:FILT %s\n",OutpFilterArr[filter]));   

	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Get Ouput Filter                                                */
/* Purpose:  This function gets the reconstruction filters to be cut into    */
/*           the output signal path.                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_GetOutFlt (ViSession instrSession, ViInt32 channel,
	ViPInt32 filter)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;
	ViChar      buffer[BUFFER_SIZE];
	ViInt32     i;
	
	if (rsamiq_invalidViInt32Range (channel, 0, 1))  
	   return VI_ERROR_PARAMETER2;
	
	if (!filter) return VI_ERROR_PARAMETER3;
	
	CHECKERR (viQueryf (instrSession, ":OUTP:%s:FILT?\n","%s", ChannelArr[channel], buffer));

	for (i = 0; OutpFilterArr[i]; i++)
		if (!strcmp(buffer, OutpFilterArr[i])) break;

	if (!OutpFilterArr[i])
		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
			
	*filter = i;

	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Set Amplitude                                                   */
/* Purpose:  This function sets the amplitude in the unbalanced output mode. */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_SetAmpl (ViSession instrSession, ViInt32 channel,
	ViReal64 amplitude)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;
	
	if (rsamiq_invalidViInt32Range (channel, 0, 1))  
	   return VI_ERROR_PARAMETER2;
	if (rsamiq_invalidViReal64Range (amplitude, 0.0, 1.0))   
	   return VI_ERROR_PARAMETER3;    
	
	CHECKERR (viPrintf (instrSession, ":OUTP:%s:AMPL %lfV\n", ChannelArr[channel],
		amplitude));
	
	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Get Amplitude                                                   */
/* Purpose:  This function gets the amplitude in the unbalanced output mode. */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_GetAmpl (ViSession instrSession, ViInt32 channel,
	ViPReal64 amplitude)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;
	
	if (rsamiq_invalidViInt32Range (channel, 0, 1))
	   return VI_ERROR_PARAMETER2;
	  
	if (!amplitude) return VI_ERROR_PARAMETER3;

	CHECKERR (viPrintf (instrSession, ":OUTP:%s:AMPL?\n", ChannelArr[channel]));
	CHECKERR (viScanf (instrSession, "%lf", amplitude));

	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Set Ouput State                                                 */
/* Purpose:  This function sets the output state.                            */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_SetOutState (ViSession instrSession, ViInt32 channel,
	ViInt32 state)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;
	
	if (rsamiq_invalidViInt32Range (channel, 0, 1))  
	   return VI_ERROR_PARAMETER2;
	if (rsamiq_invalidViInt32Range (state, 0, 3))    
	   return VI_ERROR_PARAMETER3;    
	
	CHECKERR (viPrintf (instrSession, ":OUTP:%s %s\n", ChannelArr[channel],
		OutpStateArr[state]));  
	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}
/*===========================================================================*/
/* Function: Get Ouput State                                                 */
/* Purpose:  This function gets the output state.                            */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_GetOutState (ViSession instrSession, ViInt32 channel,
	ViPInt32 state)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;
	ViChar      buffer[BUFFER_SIZE];
	ViInt32     i;
	
	if (rsamiq_invalidViInt32Range (channel, 0, 1))  
	   return VI_ERROR_PARAMETER2;
	
	if(!state) return VI_ERROR_PARAMETER3;

	CHECKERR (viPrintf (instrSession, ":OUTP:%s:STAT?\n", ChannelArr[channel]));
	CHECKERR (viScanf (instrSession, "%s", buffer));

	for (i = 0; OutpStateArr[i]; i++)
			if (!strcmp(buffer, OutpStateArr[i])) break;
			
	if (!OutpStateArr[i])
			return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	*state = i;

	CHECKERR (rsamiq_checkStatus (instrSession));

	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Set Impedance                                                   */
/* Purpose:  This function sets the impedance of the AMIQ outputs when       */
/*           switched off.                                                   */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_SetImpedance (ViSession instrSession, ViBoolean impedance)
{
	ViStatus    rsamiq_status = VI_SUCCESS;
	
	if (rsamiq_invalidViBooleanRange (impedance))    
	   return VI_ERROR_PARAMETER2;
	
	CHECKERR (viPrintf (instrSession, ":OUTP:OIMP %s\n", OutpImpArr[impedance]));  
	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Get Impedance                                                   */
/* Purpose:  This function gets the impedance of the AMIQ outputs when       */
/*           switched off.                                                   */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_GetImpedance (ViSession instrSession, ViPBoolean impedance)
{
	ViStatus    rsamiq_status = VI_SUCCESS;
	ViChar      buffer[BUFFER_SIZE];
	ViUInt16    i;
	
	if (!impedance) return VI_ERROR_PARAMETER2;

	CHECKERR (viQueryf (instrSession, ":OUTP:OIMP?\n", "%s", buffer));

	for (i = 0; OutpImpArr[i]; i++)
			if (!strcmp(buffer, OutpImpArr[i])) break;
	if (!OutpImpArr[i])
			return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	*impedance = i;

	CHECKERR (rsamiq_checkStatus (instrSession));

	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Set Ref Osc Source                                              */
/* Purpose:  This function sets the reference oscillator source.             */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_SetRefOscSrc (ViSession instrSession, ViBoolean source)
{
	ViStatus    rsamiq_status = VI_SUCCESS;

	if (rsamiq_invalidViBooleanRange (source))   
	   return VI_ERROR_PARAMETER2;

	CHECKERR (viPrintf (instrSession, ":ROSC:SOUR %s\n", RefOscArr[source]));  
	CHECKERR (rsamiq_checkStatus (instrSession));

	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Get Ref Osc Source                                              */
/* Purpose:  This function gets the reference oscillator source.             */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_GetRefOscSrc (ViSession instrSession, ViPBoolean source)
{
	ViStatus    rsamiq_status = VI_SUCCESS;
	ViChar      buffer[BUFFER_SIZE];
	ViUInt16    i;

	if (!source) return VI_ERROR_PARAMETER2;

	CHECKERR (viQueryf (instrSession, "ROSC:SOUR?\n", "%s", buffer));

	for (i = 0; RefOscArr[i]; i++)
		if (!strcmp(buffer, RefOscArr[i])) break;

	if (!RefOscArr[i])
			return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	*source = i;

	CHECKERR (rsamiq_checkStatus (instrSession));

	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Set Output Resolution                                           */
/* Purpose:  This function sets resolution of the analog outputs.            */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_SetOutRes (ViSession instrSession, ViInt32 resolution)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;
	
	if (rsamiq_invalidViInt32Range (resolution, 8, 16))  
	   return VI_ERROR_PARAMETER2;
	 
	CHECKERR (viPrintf (instrSession, ":OUTP:RES %ld\n", resolution));
	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Get Output Resolution                                           */
/* Purpose:  This function gets resolution of the analog outputs.            */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_GetOutRes (ViSession instrSession, ViPInt32 resolution)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;
	
	if (!resolution) return VI_ERROR_PARAMETER2;

	CHECKERR (viPrintf (instrSession, ":OUTP:RES?\n"));
	CHECKERR (viScanf (instrSession, "%ld", resolution));

	CHECKERR (rsamiq_checkStatus (instrSession));

	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Set Bias                                                        */
/* Purpose:  This function sets the DC offset (bias voltage) for I or Q      */
/*           channels in the balanced mode.                                  */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_SetBias (ViSession instrSession, ViInt32 channel,
	ViReal64 bias)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;
	
	if (rsamiq_invalidViInt32Range (channel, 0, 1))  
	   return VI_ERROR_PARAMETER2;
	if (rsamiq_invalidViReal64Range (bias, -2.5, 2.5))   
	   return VI_ERROR_PARAMETER3;    
	 
	CHECKERR (viPrintf (instrSession, ":OUTP:%s:BIAS %lfV\n", ChannelArr[channel],
		bias));  
	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Get Bias                                                        */
/* Purpose:  This function gets the DC offset (bias voltage) for I or Q      */
/*           channels in the balanced mode.                                  */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_GetBias (ViSession instrSession, ViInt32 channel,
	ViPReal64 bias)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;
	
	if (rsamiq_invalidViInt32Range (channel, 0, 1))  
	   return VI_ERROR_PARAMETER2;
	   
	if (!bias) return VI_ERROR_PARAMETER3;

	CHECKERR (viPrintf (instrSession, ":OUTP:%s:BIAS?\n", ChannelArr[channel]));
	CHECKERR (viScanf (instrSession, "%lf", bias));

	CHECKERR (rsamiq_checkStatus (instrSession));

	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Set Ouput Type                                                  */
/* Purpose:  This function switches between the outputs I and Q referred to  */
/*           ground (unbalanced) and the differential outputs I and Ineg, Q  */
/*           and Qneg (balanced).                                            */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_SetOutType (ViSession instrSession, ViBoolean type)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;
	
	if (rsamiq_invalidViBooleanRange (type))     
	   return VI_ERROR_PARAMETER2;
	   
	CHECKERR (viPrintf (instrSession, ":OUTP:TYPE %s\n", OutpTypeArr[type]));
	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Get Ouput Type                                                  */
/* Purpose:  This function gets the output type (balanced/unbalanced).       */      
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_GetOutType (ViSession instrSession, ViPBoolean type)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;
	ViChar      buffer[BUFFER_SIZE];
	ViUInt16    i;
	
	if (!type) return VI_ERROR_PARAMETER2;

	CHECKERR (viQueryf (instrSession, ":OUTP:TYPE?\n", "%s", buffer));

	for (i = 0; OutpTypeArr[i]; i++)
		if (!strcmp(buffer, OutpTypeArr[i])) break;
			
	if (!OutpTypeArr[i])
		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
				
	*type = i;

	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Set Amplitude (Balanced)                                        */
/* Purpose:  This function sets the amplitude in the balanced output mode.   */          
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_SetAmplBal (ViSession instrSession, ViInt32 channel,
	ViReal64 amplitude)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;
	
	if (rsamiq_invalidViInt32Range (channel, 0, 1))  
	   return VI_ERROR_PARAMETER2;
	if (rsamiq_invalidViReal64Range (amplitude, 0.0, 4.0))   
	   return VI_ERROR_PARAMETER3;  
	   
	CHECKERR (viPrintf (instrSession, ":OUTP:%s:AMPL:BAL %lfV\n", ChannelArr[channel],
		amplitude));
	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Get Amplitude (Balanced)                                        */
/* Purpose:  This function gets the amplitude in the balanced output mode.   */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_GetAmplBal (ViSession instrSession, ViInt32 channel,
	ViPReal64 amplitude)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;
	
	if (rsamiq_invalidViInt32Range (channel, 0, 1))  
	   return VI_ERROR_PARAMETER2;
	   
	if (!amplitude) return VI_ERROR_PARAMETER3;
	
	CHECKERR (viPrintf (instrSession, ":OUTP:%s:AMPL:BAL?\n", ChannelArr[channel]));
	CHECKERR (viScanf (instrSession, "%lf", amplitude));
	
	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Set Digital Output                                              */
/* Purpose:  The function ON switches on the 16-bit wide digital I/Q outputs */
/*           if pin 66 of the 68-contact SCSI connector is at HIGH.          */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_SetDigiOut (ViSession instrSession, ViBoolean state)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;

	if (rsamiq_invalidViBooleanRange (state))
	   return VI_ERROR_PARAMETER2;

	CHECKERR (viPrintf (instrSession, ":OUTP:DIG %s\n", (state == VI_TRUE) ? "ON" : "OFF"));
	CHECKERR (rsamiq_checkStatus (instrSession));

	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Get Digital Output                                              */
/* Purpose:  This function returns whether the digital output is switched to */
/*           On or Off.                                                      */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_GetDigiOut (ViSession instrSession, ViPBoolean state)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;
	ViChar      buffer[BUFFER_SIZE];
	ViUInt16        i;
	
	if (!state) return VI_ERROR_PARAMETER2;    
	
	CHECKERR (viQueryf (instrSession, ":OUTP:DIG?\n", "%s", buffer));
	
	for (i = 0; OutpClockArr[i]; i++)
		if (!strcmp(buffer, OutpClockArr[i])) break;
	if (!OutpClockArr[i])
		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
			
	*state = i;

	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Set Ext Clock                                                   */
/* Purpose:  The function sets the ext clock.                                */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_SetExtClk (ViSession instrSession, ViInt32 mode)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;

	if (rsamiq_invalidViInt32Range (mode, 0, 2))
	   return VI_ERROR_PARAMETER2;

	CHECKERR (viPrintf (instrSession, ":SCL %s\n", OutpExtClock[mode]));
	CHECKERR (rsamiq_checkStatus (instrSession));

	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Get Ext Clock                                                   */
/* Purpose:  This function returns the actual setting of the ext clock.      */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_GetExtClk (ViSession instrSession, ViPInt32 mode)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;
	ViChar      buffer[BUFFER_SIZE];
	ViUInt16    i;
	
	if (!mode) return VI_ERROR_PARAMETER2;    
	
	CHECKERR (viQueryf (instrSession, ":SCL?\n", "%s", buffer));
	
	for (i = 0; OutpExtClock[i]; i++)
		if (!strcmp(buffer, OutpExtClock[i])) break;
	if (!OutpExtClock[i])
		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
			
	*mode = i;

	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Set trigger slope                                               */
/* Purpose:  This function sets the trigger slope.                           */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_SetTriggSlope (ViSession instrSession,ViBoolean slope)
{                               
	ViStatus rsamiq_status = VI_SUCCESS;
	
	if (rsamiq_invalidViBooleanRange (slope))    
	   return VI_ERROR_PARAMETER2;
   
	CHECKERR (viPrintf (instrSession, ":TRIG:SLOP %s\n", TriggSlopArr[slope]));   
	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Get trigger slope                                               */
/* Purpose:  This function returns the trigger slope.                        */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_GetTriggSlope (ViSession instrSession, ViPBoolean slope)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;
	ViChar      buffer[BUFFER_SIZE];
	ViUInt16    i;

	if (!slope) return VI_ERROR_PARAMETER2;
	
	CHECKERR (viQueryf (instrSession, ":TRIG:SLOP?\n", "%s", buffer));
	  
	for (i = 0; TriggSlopArr[i]; i++)
		if (!strcmp(buffer, TriggSlopArr[i])) break;
			
	if (!TriggSlopArr[i])
		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
				
	*slope = i;

	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Set trigger mode                                                */
/* Purpose:  This function sets the trigger mode.                            */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_SetTriggMode (ViSession instrSession, ViInt32 mode)
{                               
	ViStatus rsamiq_status = VI_SUCCESS;
	
	if (rsamiq_invalidViInt32Range (mode, 0, 3))     
	   return VI_ERROR_PARAMETER2;
   
	CHECKERR (viPrintf (instrSession, ":TRIG:MODE %s\n", TriggModeArr[mode]));   
	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Get trigger mode                                                */
/* Purpose:  This function returns the trigger mode.                         */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_GetTriggMode (ViSession instrSession,ViPInt32 mode)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;
	ViChar      buffer[BUFFER_SIZE];
	ViInt32     i;

	if (!mode) return VI_ERROR_PARAMETER2;
	
	CHECKERR (viQueryf (instrSession, ":TRIG:MODE?\n", "%s", buffer));
	  
	for (i = 0; TriggModeArr[i]; i++)
		if (!strcmp(buffer, TriggModeArr[i])) break;
			
	if (!TriggModeArr[i])
		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
				
	*mode = i;
	
	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Set trigger source                                              */
/* Purpose:  Determines the trigger source                                   */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_SetTriggSource (ViSession instrSession,ViInt32 source)
{                               
	ViStatus rsamiq_status = VI_SUCCESS;
	
	if (rsamiq_invalidViInt32Range (source, 0, 2))   
	   return VI_ERROR_PARAMETER2;
   
	CHECKERR (viPrintf (instrSession, ":TRIG:SOUR %s\n", TriggSourceArr[source]));   
	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Get trigger source                                              */
/* Purpose:  This function returns the trigger source                        */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_GetTriggSource (ViSession instrSession, ViPInt32 source)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;
	ViChar      buffer[BUFFER_SIZE];
	ViInt32     i;

	if (!source) return VI_ERROR_PARAMETER2;
	
	CHECKERR (viQueryf (instrSession, ":TRIG:SOUR?\n", "%s", buffer));
	  
	for (i = 0; TriggSourceArr[i]; i++)
		if (!strcmp(buffer, TriggSourceArr[i])) break;
			
	if (!TriggSourceArr[i])
		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
				
	*source = i;
	
	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Calibrate                                                       */
/* Purpose:  This function performs the instrument calibration. It does not  */
/*           return until the calibration has been completed.                */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_Calibrate (ViSession instrSession, ViInt32 mode,
				ViPInt32 result)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;
	ViInt32     oldTimeout;
	
	if (rsamiq_invalidViInt32Range (mode, 0, 6))
	   return VI_ERROR_PARAMETER2;

	if(!result) return VI_ERROR_PARAMETER3;

	CHECKERR (viGetAttribute (instrSession, VI_ATTR_TMO_VALUE, &oldTimeout));
	CHECKERR (viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, AMIQ_ADJUSTMENT_TIMEOUT));

	CHECKERR (viQueryf(instrSession, ":CAL:%s\n", "%d", CalibrationArr[mode], result));
	CHECKERR (rsamiq_checkStatus (instrSession));

	CHECKERR (viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, oldTimeout));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: BERT Control                                                    */
/* Purpose:  This function controls the BETR measurement.                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_BERTCtrl (ViSession instrSession, ViBoolean mode)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;
	
	if (rsamiq_invalidViBooleanRange (mode))     
	   return VI_ERROR_PARAMETER2;
	   
	CHECKERR (viPrintf (instrSession, ":BERT:%s\n", BERTCtrlArr[mode]));
	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Trigger Abort                                                   */
/* Purpose:  Interupts the current output                                    */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_TriggAbort (ViSession instrSession)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;
	   
	CHECKERR (viPrintf (instrSession, ":ABOR\n"));
	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Trigger Arm                                                     */
/* Purpose:  Activates the output buffer which waits for a trigger signal.   */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_TriggArm (ViSession instrSession)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;
	   
	CHECKERR (viPrintf (instrSession, ":ARM\n"));
	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Trigger Immediate                                               */
/* Purpose:  Displays the trigger event                                      */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_TriggImmediate (ViSession instrSession)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;
	   
	CHECKERR (viPrintf (instrSession, ":TRIG\n"));
	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Waveform To RAM                                                 */
/* Purpose:  Transfer user-defined I/Q waveform directly to the AMIQ RAM.    */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_WaveformToRAM (ViSession instrSession,
							   ViPReal64 I, ViPReal64 Q,ViInt32 size,
							   ViPUInt32 bytesWritten)

{
	ViStatus    rsamiq_status = VI_SUCCESS;
	ViUInt16    interface = VI_INTF_GPIB;    
	ViInt32     headerSize, file_size;
	ViInt32     i, oldTimeout;
	ViPUInt16   tempBuffer;
	ViPUInt16   p2Buffer;
	ViPChar     p2CBuffer;
	ViChar      header[128];
	ViReal64    flopySpeed = 15.0;
	rsamiq_instrRange instrPtr;
	
	if (!I) return VI_ERROR_PARAMETER2;
	if (!Q) return VI_ERROR_PARAMETER3;
	if (0>=size) return VI_ERROR_PARAMETER4;
	if(!bytesWritten) return VI_ERROR_PARAMETER5;
	
	(*bytesWritten) = 0;
	
	CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
	flopySpeed = instrPtr -> flopySpeed;
	
	/* Create waveform header */
	sprintf (header, "\n{TYPE: WV, 0}\n{WAVEFORM-%d: 0,#", size*2*sizeof(ViUInt16)+3);
	
	/* Allocated temp buffer */
	tempBuffer=(ViPUInt16)malloc(2*size*sizeof(ViUInt16)+256+strlen(header)+2);
	
	/* Make Header */
	p2CBuffer=(ViPChar)tempBuffer;
	sprintf(p2CBuffer,":MEM:DATA RAM, #");
	p2CBuffer+=strlen(p2CBuffer);
	sprintf(p2CBuffer+1,"%d",size*2*sizeof(ViUInt16)+strlen(header)+1);
	(*(p2CBuffer)) = strlen(p2CBuffer+1)+'0';
	p2CBuffer+=strlen(p2CBuffer);
	sprintf (p2CBuffer, "%s", header);
	p2CBuffer+=strlen(header);
	p2Buffer=(ViPUInt16)p2CBuffer;
	
	/* get header size */
	headerSize=strlen((ViPChar)tempBuffer);

	for(i=0;i<size;i++){
		/* Update buffer */
		(*p2Buffer++)=((ViUInt16)(32768.0+(I[i]*32000.0)+0.5))&0xfffc;
		(*p2Buffer++)=((ViUInt16)(32768.0+(Q[i]*32000.0)+0.5))&0xfffc;  
	}

	p2CBuffer=(ViPChar)tempBuffer;
	file_size = 2*size*sizeof(ViUInt16)+headerSize;
	p2CBuffer[file_size] = '}';
	/*Add linefeed to end of buffer*/
	p2CBuffer[file_size+1] = '\n';
	CHECKERR( viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &interface));
	if (interface == VI_INTF_ASRL)
		{
		CHECKERR( viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_NONE));
		CHECKERR( viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_FALSE));
		}

	CHECKERR (viGetAttribute (instrSession, VI_ATTR_TMO_VALUE, &oldTimeout));
	
	/* Using FD TMO due to speed of comm interface. */
	CHECKERR (viSetAttribute (instrSession, VI_ATTR_TMO_VALUE,
		(ViUInt32)MAX_AMIQ_FLOPPY_FILE_LOAD_TIMEOUT));

	/* Write buffer */
	if (rsamiq_status = (viWrite (instrSession, (ViBuf)tempBuffer,
						 file_size+2, bytesWritten)),
						 rsamiq_status < VI_SUCCESS)
	{
		free(tempBuffer);
		if (interface == VI_INTF_ASRL)  /* Serial Port */
			{
			viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
			viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
			}
		return rsamiq_status;
	}
//  bytesWritten--;         /*The linefeed shouldn't be add to written bytes*/
  (*bytesWritten)--;         // changed 05-Sep-2003 og
	/* Free temp buffer */
	free(tempBuffer);
	if (interface == VI_INTF_ASRL)  /* Serial Port */
		{
		viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
		viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
		}

	CHECKERR (rsamiq_checkStatus (instrSession));

	CHECKERR (viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, oldTimeout));    
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Waveform File To RAM                                            */
/* Purpose:  This function transfers the "*.WV" file from the host computer  */
/*           into the AMIQ's RAM.                                            */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_WaveformFileToRAM (ViSession instrSession,
											ViString name)

{
	ViStatus    rsamiq_status = VI_SUCCESS;
	ViReal64    speed = 400.0;
	ViUInt16    interface = VI_INTF_GPIB;    
	ViInt32     headerSize;
	ViPChar     tempBuffer;
	ViPUInt16   p2Buffer;
	ViPChar     p2CBuffer;
	FILE        *sourceFile;
	ViInt32     file_size;  
	ViUInt32    bytesWritten;   
	ViUInt32    bytesRead;
	ViInt32     newTimeout = 10000,
				oldTimeout, i;
	rsamiq_instrRange instrPtr;

	if (!name) return VI_ERROR_PARAMETER2;
	
	CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
	speed = instrPtr -> speed;

	/* Open file */
	if ((sourceFile = fopen (name, "rb")) == VI_NULL)
		return VI_ERROR_INSTR_FILE_OPEN; /* not defined by VTL */

	/* seek for size */
	if(fseek (sourceFile, 0, SEEK_END)) VI_ERROR_INSTR_FILE_OPEN;
	file_size=ftell(sourceFile);
	if(fseek (sourceFile, 0, SEEK_SET)) VI_ERROR_INSTR_FILE_OPEN;
	
	/* Allocate temp buffer (+1 for linefeed)*/
	tempBuffer=(ViPChar)malloc(file_size+256+1);

	/* Make Header */
	p2CBuffer=tempBuffer;
	sprintf(p2CBuffer,":MEM:DATA RAM, #");
	p2CBuffer+=strlen(p2CBuffer);
	sprintf(p2CBuffer+1,"%d",file_size);
	(*(p2CBuffer))=strlen(p2CBuffer+1)+'0';
	p2CBuffer+=strlen(p2CBuffer);
	p2Buffer=(ViPUInt16)p2CBuffer;
	
	/* get header size */
	headerSize=strlen(tempBuffer);

	/* Read file */
	bytesRead=fread (p2Buffer, 1, file_size, sourceFile);

	/* Init p2CBuffer */
	p2CBuffer=tempBuffer;

	for (i = 0; aDrive[i]; i++)
		if (strstr(name, aDrive[i]))
			{
			ViReal64 flopySpeed;
			flopySpeed = instrPtr -> flopySpeed;
			newTimeout = (ViInt32)MAX_AMIQ_FLOPPY_FILE_LOAD_TIMEOUT;
			break;
			}
		else
			newTimeout = (ViInt32)MAX_AMIQ_HD_FILE_LOAD_TIMEOUT;

	CHECKERR( viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &interface));
	if (interface == VI_INTF_ASRL)
		{
		CHECKERR( viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_NONE));
		CHECKERR( viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_FALSE));
		}

	CHECKERR (viGetAttribute (instrSession, VI_ATTR_TMO_VALUE, &oldTimeout));
	CHECKERR (viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, newTimeout));

	/* Write buffer */
	tempBuffer[bytesRead+headerSize] = '\n';
	if (rsamiq_status = (viWrite (instrSession, (ViBuf)tempBuffer,
						 bytesRead+headerSize+1,&bytesWritten)),
						 rsamiq_status < VI_SUCCESS)
	{
		free(tempBuffer);
		if (interface == VI_INTF_ASRL)  /* Serial Port */
			{
			viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
			viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
			}
		return rsamiq_status;
	}
	bytesWritten--;     /*The linefeed shouldn't be add to written bytes*/
	/* Free temp buffer */
	free(tempBuffer);
	/* Close File */
	fclose(sourceFile);
	
	if (interface == VI_INTF_ASRL)  /* Serial Port */
		{
		viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
		viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
		}
	
	
	CHECKERR (rsamiq_checkStatus (instrSession));

	CHECKERR (viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, oldTimeout));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Waveform To Disk                                                */
/* Purpose:  Transfer user-defined I/Q waveform data directly to the AMIQ's  */
/*           disk drive.                                                     */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_WaveformToDisk (ViSession instrSession,
										 ViReal64 _VI_FAR I[], ViReal64 _VI_FAR Q[],
										 ViInt32 size, ViString filename,
										 ViPUInt32 bytesWritten)
{
	ViStatus    rsamiq_status = VI_SUCCESS;
	ViUInt16    interface = VI_INTF_GPIB;
	ViInt32     headerSize;
	ViInt32     i;
	ViPChar     tempBuffer;
	ViPUInt16   p2Buffer;
	ViPChar     p2CBuffer;
	ViChar      header[128];
	ViInt32     newTimeout = 10000,
				oldTimeout,
				file_size;
	ViReal64    speed = 400.0;
	rsamiq_instrRange instrPtr;
	
	if (!I) return VI_ERROR_PARAMETER2;
	if (!Q) return VI_ERROR_PARAMETER3;
	if (0>=size) return VI_ERROR_PARAMETER4;
	if (!filename) return VI_ERROR_PARAMETER5;
	if (!bytesWritten) return VI_ERROR_PARAMETER6;

	CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
	speed = instrPtr -> speed;

	(*bytesWritten) = 0;
	
	/* Create waveform header */
	sprintf (header, "\n{TYPE: WV, 0}\n{WAVEFORM-%d: 0,#", size*2*sizeof(ViUInt16)+3);
	
	/* Allocated temp buffer */
	tempBuffer=(ViPChar)malloc(2*size*sizeof(ViUInt16)+256+strlen(header)+2);
	
	/* Make Header */
	p2CBuffer=tempBuffer;
	sprintf(p2CBuffer,":MMEM:DATA \"%s\", #", filename);
	p2CBuffer+=strlen(p2CBuffer);
	sprintf(p2CBuffer+1,"%d",size*2*sizeof(ViUInt16)+strlen(header)+1);
	(*(p2CBuffer))=strlen(p2CBuffer+1)+'0';
	p2CBuffer+=strlen(p2CBuffer);
	sprintf (p2CBuffer, "%s", header);
	p2CBuffer+=strlen(header);
	p2Buffer=(ViPUInt16)p2CBuffer;
	
	/* get header size */
	headerSize=strlen(tempBuffer);

	for(i=0;i<size;i++){
		/* Update buffer */
		(*p2Buffer++)=((ViUInt16)(32768.0+(I[i]*32000.0)+0.5))&0xfffc;
		(*p2Buffer++)=((ViUInt16)(32768.0+(Q[i]*32000.0)+0.5))&0xfffc;  
	}

	p2CBuffer=tempBuffer;
	file_size = 2*size*sizeof(ViUInt16)+headerSize;
	p2CBuffer[file_size] = '}';
	p2CBuffer[file_size+1] = '\n';
	
	for (i = 0; aDrive[i]; i++)
		if (strstr(filename, aDrive[i]))
			{
			ViReal64 flopySpeed;
			flopySpeed = instrPtr -> flopySpeed;
			newTimeout = (ViInt32)MAX_AMIQ_FLOPPY_FILE_LOAD_TIMEOUT;
			break;
			}
		else
			newTimeout = (ViInt32)MAX_AMIQ_HD_FILE_LOAD_TIMEOUT;

	CHECKERR( viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &interface));
	if (interface == VI_INTF_ASRL)
		{
		CHECKERR( viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_NONE));
		CHECKERR( viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_FALSE));
		}

	CHECKERR (viGetAttribute (instrSession, VI_ATTR_TMO_VALUE, &oldTimeout));
	CHECKERR (viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, newTimeout));
	/* Write buffer */
	if (rsamiq_status = (viWrite (instrSession, (ViBuf)tempBuffer,
						 file_size+2, bytesWritten)),
						 rsamiq_status < VI_SUCCESS)
	{
		free(tempBuffer);
		if (interface == VI_INTF_ASRL)  /* Serial Port */
			{
			viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
			viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
			}
		return rsamiq_status;
	}
	bytesWritten--;             /*The linefeed shouldn't be add to written bytes*/
	/* Free temp buffer */
	free(tempBuffer);
	
	if (interface == VI_INTF_ASRL)  /* Serial Port */
		{
		viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
		viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
		}

	CHECKERR (rsamiq_checkStatus (instrSession));
	CHECKERR (viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, oldTimeout));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Waveform File To Disk                                           */
/* Purpose:  This function stores specified "*.WV" file directly on the      */
/*           AMIQ's disk drive.                                              */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_WaveformFileToDisk (ViSession instrSession,
											 ViString source, ViString destination)
{
	ViStatus    rsamiq_status = VI_SUCCESS;
	ViUInt16    interface = VI_INTF_GPIB;
	ViInt32     headerSize;
	ViPChar   tempBuffer;
	ViPUInt16   p2Buffer;
	ViPChar     p2CBuffer;
	FILE        *sourceFile;
	ViInt32     file_size, i,
				newTimeout = 10000,
				oldTimeout;
	ViUInt32    bytesWritten;   
	ViUInt32    bytesRead;
	ViReal64    speed;
	rsamiq_instrRange instrPtr;
	
	if (!source) return VI_ERROR_PARAMETER2;
	if (!destination) return VI_ERROR_PARAMETER3;

	CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
	speed = instrPtr -> speed;

	/* Open file */
	if ((sourceFile = fopen (source, "rb")) == VI_NULL)
		return VI_ERROR_INSTR_FILE_OPEN; /* not defined by VTL */

	/* seek for size */
	if(fseek (sourceFile, 0, SEEK_END)) VI_ERROR_INSTR_FILE_OPEN;
	file_size=ftell(sourceFile);
	if(fseek (sourceFile, 0, SEEK_SET)) VI_ERROR_INSTR_FILE_OPEN;
	
	/* Allocate temp buffer */
	tempBuffer=(ViPChar)malloc(file_size+256);

	/* Make Header */
	p2CBuffer=tempBuffer;
	sprintf(p2CBuffer,":MMEM:DATA \"%s\", #",destination);
	p2CBuffer+=strlen(p2CBuffer);
	sprintf(p2CBuffer+1,"%d",file_size);
	(*(p2CBuffer))=strlen(p2CBuffer+1)+'0';
	p2CBuffer+=strlen(p2CBuffer);
	p2Buffer=(ViPUInt16)p2CBuffer;
	
	/* get header size */
	headerSize=strlen(tempBuffer);

	/* Read file */
	bytesRead=fread (p2Buffer, 1, file_size, sourceFile);

	/* Init p2CBuffer */
	p2CBuffer=tempBuffer;

	for (i = 0; aDrive[i]; i++)
		if (strstr(destination, aDrive[i]))
			{
			ViReal64 flopySpeed;
			flopySpeed = instrPtr -> flopySpeed;
			newTimeout = (ViInt32)MAX_AMIQ_FLOPPY_FILE_LOAD_TIMEOUT;
			break;
			}
		else
			newTimeout = (ViInt32)MAX_AMIQ_HD_FILE_LOAD_TIMEOUT;

	CHECKERR( viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &interface));
	if (interface == VI_INTF_ASRL)
		{
		CHECKERR( viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_NONE));
		CHECKERR( viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_FALSE));
		}


	CHECKERR (viGetAttribute (instrSession, VI_ATTR_TMO_VALUE, &oldTimeout));
	CHECKERR (viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, newTimeout));

	tempBuffer[bytesRead+headerSize] = '\n';
	
	/* Write buffer */
	
	if (rsamiq_status = (viWrite (instrSession, (ViBuf)tempBuffer,
						 bytesRead+headerSize+1, &bytesWritten)),
						 rsamiq_status < VI_SUCCESS)
	{
		free(tempBuffer);
		if (interface == VI_INTF_ASRL)  /* Serial Port */
			{
			viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
			viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
			}
			
		return rsamiq_status;
	}
	
	bytesWritten--;         /*The linefeed shouldn't be add to written bytes*/
	/* Free temp buffer */
	free(tempBuffer);
	
	/* Close File */
	fclose(sourceFile);

	if (interface == VI_INTF_ASRL)  /* Serial Port */
		{
		viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, VI_ASRL_END_TERMCHAR);
		viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, VI_TRUE);
		}

	CHECKERR (rsamiq_checkStatus (instrSession));

	CHECKERR (viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, oldTimeout));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Setup Dir                                                       */
/* Purpose:  This function lists the contens of the Setup directory          */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_SetupDir (ViSession instrSession,
									ViChar _VI_FAR fileList[], ViInt32 bufferSize,
									ViPInt32 numberOfItems)
{                               
	ViStatus rsamiq_status = VI_SUCCESS;
	ViPChar p2Buffer = fileList;
	ViInt32 i;
	ViBoolean firstTurn = 1;
	ViBoolean nextValid;
	
	if (!fileList) return VI_ERROR_PARAMETER2;
	if (!numberOfItems) return VI_ERROR_PARAMETER4;
	
	(*numberOfItems) = 0;
	/*  list buffer is empty */
	if (!fileList)
		{
		CHECKERR (viQueryf (instrSession, ":SYST:STATE:CAT:LENG?\n", "%ld", numberOfItems));  
		CHECKERR (rsamiq_checkStatus (instrSession));   
		return rsamiq_status;
		}

	/* Send command */
	CHECKERR (viPrintf (instrSession, ":SYST:STAT:CAT?\n"));
   
	/* Read response */
	CHECKERR (viRead (instrSession, (ViBuf)fileList, bufferSize, (ViPUInt32)&bufferSize)); 

	if(bufferSize){
	
		/* Parse response buffer */     
		for(i=0;i<bufferSize;i++){
		
			if(fileList[i]=='\n'){
				(*numberOfItems)++;
				break;
			}

			if (!firstTurn && (i+1)!=bufferSize)
				nextValid = (ViBoolean)(fileList[i+1] == '"');
				else nextValid = 0;
			
			if(fileList[i] == ',' && (firstTurn || nextValid )) {
				firstTurn = 0;
				fileList[i]='\n';
				p2Buffer=&fileList[i];
				(*numberOfItems)++;
			}


		}   
			
		if(i==bufferSize) *p2Buffer=0; 
			else fileList[i]=0;
	}

	/* Remove "used bytes" and "free bytes" items from count */
	(*numberOfItems)-=2;
	
	/* Coerce numberOfItems variable */
	if((*numberOfItems)<0) (*numberOfItems) = 0;
	
	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
	
}

/*===========================================================================*/
/* Function: Mem Copy                                                        */
/* Purpose:  This function copies source file into destination.              */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_SetupCopy (ViSession instrSession, ViString source,
									ViString destination)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;

	if (!source) return VI_ERROR_PARAMETER2;
	if (!destination) return VI_ERROR_PARAMETER4;
	
	CHECKERR (viPrintf (instrSession, ":SYST:STAT:COPY '%s','%s'\n", source, destination));
	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Mem Delete                                                      */
/* Purpose:  This function deletes the specified file.                       */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_SetupDel (ViSession instrSession, ViString name)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;
	
	if (!name) return VI_ERROR_PARAMETER2;
	
	CHECKERR (viPrintf (instrSession, ":SYST:STAT:DEL \"%s\"\n", name));
	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}


/*===========================================================================*/
/* Function: Setup Recall                                                    */
/* Purpose:  This function recalls setup from file                           */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_SetupRecall (ViSession instrSession, ViString name)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;
	
	if (!name) return VI_ERROR_PARAMETER2;
	
	CHECKERR (viPrintf (instrSession, "*RCL \"%s\"\n", name));
	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Setup Save                                                      */
/* Purpose:  This function recalls setup from file                           */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_SetupSav (ViSession instrSession, ViString name)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;
	
	if (!name) return VI_ERROR_PARAMETER2;
	
	CHECKERR (viPrintf (instrSession, "*SAV \"%s\"\n", name));
	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: State Checking                                                  */
/* Purpose:  This function switches ON/OFF state checking of the instrument  */
/*           (reading of the Standard Event Register and checking it for     */
/*           error). Nearly all driver function are using state checking.    */
/*           Switch this function to ON when debug your application. For     */
/*           better bus throughput and instruments performance switch it OFF.*/
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_errorCheckState (ViSession instrSession, ViBoolean state)
{
	ViStatus rsamiq_status = VI_SUCCESS;
	rsamiq_instrRange instrPtr;

	CHECKERR( viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
	instrPtr -> errorChecking = state;

	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Set GPIB address                                                */
/* Purpose:  This function changes GPIB address                              */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_SetGPIBAddress (ViSession instrSession, ViInt32 GPIBAdr)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;

	if(rsamiq_invalidViInt32Range (GPIBAdr, 1, 30))
		return VI_ERROR_PARAMETER2;
	
	CHECKERR (viPrintf (instrSession, ":SYST:COMM:GPIB:ADDR %d\n", GPIBAdr));
	CHECKERR (rsamiq_checkStatus (instrSession));
		
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Set ASRL Speed                                                  */
/* Purpose:  This function sets transmission speed of the serial interface.  */
/*           This setting must apply to both directions. To AMIQ's side and  */
/*           also to computer's side.                                        */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_SetASRLSpeed (ViSession instrSession,
				ViInt32 baudrate)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;

	if(rsamiq_invalidViInt32Range (baudrate, 300, 115200))
		return VI_ERROR_PARAMETER2;
	
	CHECKERR (viPrintf (instrSession, ":SYST:COMM:SER:BAUD %d\n", baudrate));
	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Get System options                                              */
/* Purpose:  This function gets the system options                           */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_GetSysOpt (ViSession instrSession, ViPChar options)
{                               
	ViStatus    rsamiq_status = VI_SUCCESS;

	if(!options) return VI_ERROR_PARAMETER2;
	
	CHECKERR (viQueryf (instrSession, ":SYST:OPT?\n", "%s", options));
	CHECKERR (rsamiq_checkStatus (instrSession));
	
	return rsamiq_status;
}

/*****************************************************************************/
/*-------- INSERT USER-CALLABLE INSTRUMENT-DEPENDENT ROUTINES HERE ----------*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Write To Instrument                                             */
/* Purpose:  This function writes a command string to the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_writeInstrData (ViSession instrSession, ViString writeBuffer)
{
	ViStatus rsamiq_status = VI_SUCCESS;
	
	CHECKERR (viPrintf (instrSession, "%s\n", writeBuffer));
		
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Read Instrument Buffer                                          */
/* Purpose:  This function reads the output buffer of the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_readInstrData (ViSession instrSession, ViInt32 numberBytesToRead,
					ViChar _VI_FAR readBuffer[], ViPInt32 numBytesRead)
{
	ViStatus    rsamiq_status = VI_SUCCESS;
	ViUInt32    tmpCnt;

	if (numBytesRead)
		*numBytesRead = 0L;
		
	CHECKERR (viRead (instrSession, (ViBuf)readBuffer, numberBytesToRead, &tmpCnt));
	if (numBytesRead)
		*numBytesRead = (ViInt32)tmpCnt;
	
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Reset                                                           */
/* Purpose:  This function resets the instrument.  If the reset function     */
/*           is not supported by the instrument, this function returns       */
/*           the warning VI_WARN_NSUP_RESET.                                 */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_reset (ViSession instrSession)
{
	
	ViStatus rsamiq_status = VI_SUCCESS;

	CHECKERR (viPrintf (instrSession, "*RST\n"));
	CHECKERR (rsamiq_defaultInstrSetup (instrSession));
	CHECKERR (rsamiq_checkStatus (instrSession));

	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Self-Test                                                       */
/* Purpose:  This function executes the instrument self-test and returns     */
/*           the result.                                                     */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_self_test (ViSession instrSession, ViPInt16 testResult,
					ViChar _VI_FAR testMessage[])
{
	ViStatus    rsamiq_status = VI_SUCCESS;
	ViInt32     oldTimeout;

	if (!testResult)
		return VI_ERROR_PARAMETER2;

	CHECKERR (viGetAttribute (instrSession, VI_ATTR_TMO_VALUE, &oldTimeout));
	CHECKERR (viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, AMIQ_SELFTEST_TIMEOUT));
	CHECKERR (viQueryf (instrSession, "*TST?\n", "%hd", testResult));
		
	if (testMessage)
		strcpy (testMessage, (*testResult) ? "Self-test failed" : "Self-test passed");

	CHECKERR (viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, oldTimeout));
	
	CHECKERR (rsamiq_checkStatus (instrSession));

	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Self-Test SDRAM                                                 */
/* Purpose:  This function executes the instrument self-test SDRAM routine   */
/* and returns the result.                                                   */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_self_test_SDRAM (ViSession instrSession, ViPInt16 testResult,
					ViChar _VI_FAR testMessage[])
{
	ViStatus    rsamiq_status = VI_SUCCESS;
	ViInt32     oldTimeout;

	if (!testResult)
		return VI_ERROR_PARAMETER2;

	CHECKERR (viGetAttribute (instrSession, VI_ATTR_TMO_VALUE, &oldTimeout));
	CHECKERR (viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, AMIQ_RAMTEST_TIMEOUT));
	CHECKERR (viQueryf (instrSession, ":DIAG:SELF:SDR?\n", "%hd", testResult));
		
	if (testMessage)
		strcpy (testMessage, (*testResult) ? "Self-test failed" : "Self-test passed");

	CHECKERR (viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, oldTimeout));
	
	CHECKERR (rsamiq_checkStatus (instrSession));

	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Get Analog Board ID                                             */
/* Purpose:  This function returns the ID of the built-in analog board.      */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_getAnalogBoardID (ViSession instrSession,
				ViPInt32 boardID)
{
	ViStatus    rsamiq_status = VI_SUCCESS;

	if (!boardID) return VI_ERROR_PARAMETER2;

	CHECKERR (viQueryf (instrSession, ":DIAG:ABO:ID?\n", "%hd", boardID));
	CHECKERR (rsamiq_checkStatus (instrSession));

	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Get Test Point Value                                            */
/* Purpose:  This function returns the voltage of the selected test point.   */
/*           The voltage is measured by means of the diagnostic A/D          */
/*           converter. To do so the desired test point is set, the converter*/
/*           started and after a waiting time of 60 ms (maximum duration of a*/
/*           conversion procedure) the result is returned.                   */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_getDiagTPOVal (ViSession instrSession,
				ViInt32 testPointNumber, ViPReal64 testPointValue)
{
	ViStatus    rsamiq_status = VI_SUCCESS;

	if(rsamiq_invalidViInt32Range (testPointNumber, 0, 15))
		return VI_ERROR_PARAMETER2;
	if (!testPointValue) return VI_ERROR_PARAMETER3;

	CHECKERR (viQueryf (instrSession, ":DIAG:TPO%ld?\n", "%lf", testPointNumber,
		testPointValue));
	CHECKERR (rsamiq_checkStatus (instrSession));

	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Error Query                                                     */
/* Purpose:  This function queries the instrument error queue, and returns   */
/*           the result.                                                     */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_error_query (ViSession instrSession, ViPInt32 errorCode,
					ViChar _VI_FAR errorMessage[])
{
	ViStatus rsamiq_status = VI_SUCCESS;
	
	if (!errorCode)
		return VI_ERROR_PARAMETER2;
	if (!errorMessage)
		return VI_ERROR_PARAMETER3;

	CHECKERR (viQueryf (instrSession, ":SYST:ERR?\n", "%ld,\"%[^\"]", errorCode, 
																	errorMessage));
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Error Message                                                   */
/* Purpose:  This function translates the error return value from the        */
/*           instrument driver into a user-readable string.                  */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_error_message (ViSession instrSession, ViStatus statusCode,
					ViChar _VI_FAR errMessage[])
{
	ViStatus rsamiq_status = VI_SUCCESS;
	ViInt16 i;
	static rsamiq_tStringValPair statusDescArray[] = {
		{VI_ERROR_PARAMETER1,   "ERROR: Parameter 1 out of range"},
		{VI_ERROR_PARAMETER2,   "ERROR: Parameter 2 out of range"},
		{VI_ERROR_PARAMETER3,   "ERROR: Parameter 3 out of range"},
		{VI_ERROR_PARAMETER4,   "ERROR: Parameter 4 out of range"},
		{VI_ERROR_PARAMETER5,   "ERROR: Parameter 5 out of range"},
		{VI_ERROR_PARAMETER6,   "ERROR: Parameter 6 out of range"},
		{VI_ERROR_PARAMETER7,   "ERROR: Parameter 7 out of range"},
		{VI_ERROR_PARAMETER8,   "ERROR: Parameter 8 out of range"},
		{VI_ERROR_FAIL_ID_QUERY,"ERROR: Identification query failed"},
		{VI_ERROR_INV_RESPONSE, "ERROR: Interpreting instrument response"},
		{VI_ERROR_INSTR_FILE_OPEN,    "ERROR: Opening the specified file"},
		{VI_ERROR_INSTR_FILE_WRITE,   "ERROR: Writing to the specified file"},
		{VI_ERROR_INSTR_INTERPRETING_RESPONSE, "ERROR: Interpreting the instrument's response"},
		{VI_ERROR_TMO,                "ERROR: Operation timed out"},
		{rsamiq_ERROR_INSTRUMENT_ERROR,  "ERROR: Instrument status error"},
		{rsamiq_ERROR_INVALID_CONFIGURATION, "ERROR: Instrument configuration error"},
		{VI_NULL, VI_NULL}
	};

	rsamiq_status = viStatusDesc (instrSession, statusCode, errMessage);
	if (rsamiq_status == VI_WARN_UNKNOWN_STATUS) {
		for (i = 0; statusDescArray[i].stringName; i++) {
			if (statusDescArray[i].stringVal == statusCode) {
				strcpy (errMessage, statusDescArray[i].stringName);
				return (VI_SUCCESS);
			}
		}
		sprintf (errMessage, "Unknown Error 0x%08X", statusCode);
		return (VI_WARN_UNKNOWN_STATUS);
	}
	
	rsamiq_status = VI_SUCCESS;
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Revision Query                                                  */
/* Purpose:  This function returns the driver and instrument revisions.      */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_revision_query (ViSession instrSession,
					ViChar _VI_FAR driverRev[], ViChar _VI_FAR instrRev[])
{
	ViStatus rsamiq_status = VI_SUCCESS;

	strcpy (driverRev, RSAMIQ_REVISION);
	
	CHECKERR (viQueryf (instrSession, "*IDN?\n", "%*[^,],%*[^,],%*[^,],%[^\n]", instrRev));
		
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Close                                                           */
/* Purpose:  This function closes the instrument.                            */
/*===========================================================================*/
ViStatus _VI_FUNC rsamiq_close (ViSession instrSession)
{
	rsamiq_instrRange instrPtr;
	ViSession rmSession;
	ViStatus rsamiq_status = VI_SUCCESS;

	CHECKERR (viGetAttribute (instrSession, VI_ATTR_RM_SESSION, &rmSession));
	CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

	if (instrPtr != NULL) 
		free (instrPtr);
	
	rsamiq_status = viClose (instrSession);
	viClose (rmSession);

	return rsamiq_status;
}

/*****************************************************************************/
/*= UTILITY ROUTINES (Non-Exportable Functions) =============================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Boolean Value Out Of Range - ViBoolean                          */
/* Purpose:  This function checks a Boolean to see if it is equal to VI_TRUE */
/*           or VI_FALSE. If the value is out of range, the return value is  */
/*           VI_TRUE, otherwise the return value is VI_FALSE.                */
/*===========================================================================*/
ViBoolean rsamiq_invalidViBooleanRange (ViBoolean val)
{
	return (ViBoolean)((val != VI_FALSE && val != VI_TRUE) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Signed Integer Value Out Of Range - ViInt32                */
/* Purpose:  This function checks a long signed integer value to see if it   */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean rsamiq_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max)
{
	return (ViBoolean)((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Unsigned Integer Value Out Of Range - ViUInt32             */
/* Purpose:  This function checks a long unsigned integer value to see if it */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean rsamiq_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max)
{
	return (ViBoolean)((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (Double) Value Out Of Range - ViReal64                     */
/* Purpose:  This function checks a real (double) value to see if it lies    */  
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean rsamiq_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max)
{
	return (ViBoolean)((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Initialize Clean Up                                             */
/* Purpose:  This function is used only by the rsamiq_init function.  When   */
/*           an error is detected this function is called to close the       */
/*           open resource manager and instrument object sessions and to     */
/*           set the instrSession that is returned from rsamiq_init to       */
/*           VI_NULL.                                                        */
/*===========================================================================*/
ViStatus rsamiq_initCleanUp (ViSession openRMSession,
					ViPSession openInstrSession, ViStatus currentStatus)
{
	rsamiq_instrRange instrPtr;
	
	if (viGetAttribute (*openInstrSession, VI_ATTR_USER_DATA, &instrPtr) >= 0)
		if (instrPtr != NULL) 
			free (instrPtr);

	viClose (*openInstrSession);
	viClose (openRMSession);
	*openInstrSession = VI_NULL;
	
	return currentStatus;
}

/*===========================================================================*/
/* Function: Read To File From Instrument                                    */
/* Purpose:  This function is used to read data from the instrument and      */
/*           write it to a user specified file.                              */
/*===========================================================================*/
ViStatus rsamiq_readToFile (ViSession instrSession, ViString filename,
					ViUInt32 readBytes, ViPUInt32 retCount)
{
	ViStatus  rsamiq_status = VI_SUCCESS;
	ViByte    buffer[BUFFER_SIZE];
	ViUInt32  bytesReadInstr = 0, bytesWrittenFile = 0;
	FILE     *targetFile;

	*retCount = 0L;
	if ((targetFile = fopen (filename, "wb")) == VI_NULL)
		return VI_ERROR_INSTR_FILE_OPEN; /* not defined by VTL */

	for (;;) {
		if (readBytes > BUFFER_SIZE)
			rsamiq_status = viRead (instrSession, buffer, BUFFER_SIZE, &bytesReadInstr);
		else
			rsamiq_status = viRead (instrSession, buffer, readBytes, &bytesReadInstr);

		bytesWrittenFile = fwrite (buffer, sizeof (ViByte), (size_t)bytesReadInstr, targetFile);
		*retCount += bytesWrittenFile;
		if (bytesWrittenFile < bytesReadInstr)
			rsamiq_status = VI_ERROR_INSTR_FILE_WRITE; /* not defined by VTL */

		if ((readBytes <= BUFFER_SIZE) || (rsamiq_status <= 0) || (rsamiq_status == VI_SUCCESS_TERM_CHAR))
			break;

		readBytes -= BUFFER_SIZE;
	}

	fclose (targetFile);
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Write From File To Instrument                                   */
/* Purpose:  This function is used to read data from a user specified file   */
/*           and write it to the instrument.                                 */
/*===========================================================================*/
ViStatus rsamiq_writeFromFile (ViSession instrSession, ViString filename,
					ViUInt32 writeBytes, ViPUInt32 retCount)
{
	ViStatus  rsamiq_status = VI_SUCCESS;
	ViByte    buffer[BUFFER_SIZE+1];      /*The last character is for linefeed*/
	ViUInt32  bytesRead = 0, bytesWritten = 0;
	FILE     *sourceFile;
	ViBoolean sendEnd = VI_FALSE;
	ViInt32   tmpBytes;  

	*retCount = 0L;
	if ((sourceFile = fopen (filename, "rb")) == VI_NULL)
		return VI_ERROR_INSTR_FILE_OPEN; /* not defined by VTL */

	while (!feof (sourceFile)) {
		bytesRead = (ViUInt32)fread (buffer, sizeof (ViByte), BUFFER_SIZE, sourceFile);
		if ((writeBytes > BUFFER_SIZE) && (bytesRead == BUFFER_SIZE)) {
			viGetAttribute (instrSession, VI_ATTR_SEND_END_EN, &sendEnd);
			viSetAttribute (instrSession, VI_ATTR_SEND_END_EN, VI_FALSE);
			/*Add linefeed to the end of buffer*/
			buffer[BUFFER_SIZE] = '\n';
			rsamiq_status = viWrite (instrSession, buffer, BUFFER_SIZE+1, &bytesWritten);
			viSetAttribute (instrSession, VI_ATTR_SEND_END_EN, sendEnd);
			writeBytes -= BUFFER_SIZE;
			*retCount += (bytesWritten - 1);    
			if (rsamiq_status < 0)
				break;
		}
		else {
			tmpBytes = ((bytesRead < writeBytes) ? bytesRead : writeBytes);
			/*Add linefeed to the end of buffer*/   
			buffer[tmpBytes] = '\n';
			rsamiq_status = viWrite (instrSession, buffer, tmpBytes+1, &bytesWritten);
			*retCount += (bytesWritten - 1); /*The linefeed shouldn't be add to written bytes*/  
			break;
		}
	}
	fclose (sourceFile);
   
	return rsamiq_status;
}

/*****************************************************************************/
/*----------- INSERT INSTRUMENT-DEPENDENT UTILITY ROUTINES HERE -------------*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Default Instrument Setup                                        */
/* Purpose:  This function sends a default setup to the instrument.  This    */
/*           function is called by the rsamiq_reset operation and by the     */
/*           rsamiq_init function if the reset option has not been           */
/*           selected.  This function is useful for configuring any          */
/*           instrument settings that are required by the rest of the        */
/*           instrument driver functions such as turning headers ON or OFF   */
/*           or using the long or short form for commands, queries, and data.*/                                    
/*===========================================================================*/
ViStatus rsamiq_defaultInstrSetup (ViSession instrSession)
{
	ViStatus    rsamiq_status = VI_SUCCESS;
	ViUInt32    baudRate = 9600;
	ViUInt16    interface = VI_INTF_GPIB;
	rsamiq_instrRange instrPtr;
	
	/* Determine if the structure has been initialized for the current VISA  */
	/* Session and malloc if it has not.                                     */
	CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

	if (instrPtr == NULL) 
		instrPtr = malloc (sizeof (struct rsamiq_statusDataRanges));

	instrPtr -> errorChecking = VI_TRUE;
	CHECKERR( viGetAttribute (instrSession, VI_ATTR_INTF_TYPE, &interface));
	
	if (interface == VI_INTF_ASRL)  /* Serial Port */
		CHECKERR( viGetAttribute (instrSession, VI_ATTR_ASRL_BAUD, &baudRate));

	instrPtr -> speed = (interface == VI_INTF_GPIB) ? 400.0 : (ViReal64)baudRate/10000.0;
	instrPtr -> flopySpeed = (interface == VI_INTF_GPIB) ? 15.0 : ((instrPtr -> speed > 15.0) ? 15.0 : instrPtr -> speed);
	strcpy (instrPtr -> instrDriverRevision, RSAMIQ_REVISION);
	
	CHECKERR( viSetAttribute (instrSession, VI_ATTR_USER_DATA, (ViAttrState) instrPtr));
	CHECKERR( viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, AMIQ_DEFAULT_TIMEOUT));
	CHECKERR( viPrintf (instrSession, "*CLS;*SRE 32;*ESE 1\n"));
		
	return rsamiq_status;
}

/*===========================================================================*/
/* Function: Wait for OPC                                                    */
/* Purpose:  This function sends a command to the instrument and waits until */
/*           the instrument response is available (queries MAV in STB).      */
/*===========================================================================*/
ViStatus rsamiq_waitOPC (ViSession instrSession, ViPString command, ViPString instrResp,
				ViInt32 timeout)
{
	ViStatus rsamiq_status = VI_SUCCESS;
	ViUInt16 response = 0;
	ViInt32 counter = 0;

	if ((rsamiq_status = viPrintf (instrSession, "%s\n", command)) < 0)
		return rsamiq_status;

	while (!(response & 0x10)) { /* (queries MAV in STB) */

		CHECKERR (rsamiq_status = viReadSTB (instrSession, &response));

		rsamiq_Wait(0.005);
		counter+= 5;
		if (counter >= timeout)
			return VI_ERROR_TMO;
	}

	CHECKERR (rsamiq_status = viScanf (instrSession, "%s", instrResp));

	return rsamiq_status;
}

/*=========================================================================*/
/* Function: Check Status                                                  */
/* Purpose:  This function reads the system status byte.                   */
/*=========================================================================*/
ViStatus rsamiq_checkStatus (ViSession instrSession)
{
	ViStatus rsamiq_status = VI_SUCCESS;
	ViInt32 esrByte;
	rsamiq_instrRange instrPtr;

	CHECKERR( viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

	if (instrPtr -> errorChecking) { /* Is Error Checking Enabled? */
		/*--------------- Read the Standard Event Register --------------------*/
		if ((rsamiq_status = viQueryf (instrSession, "*ESR?\n", "%d", &esrByte)) < 0)
			return rsamiq_status;

		/*---------------- Check if any error or message bit was asserted -------*/
		if ((esrByte & 0x3C) != 0)
			rsamiq_status = rsamiq_ERROR_INSTRUMENT_ERROR;
		else
		   rsamiq_status = VI_SUCCESS;
	}

	return rsamiq_status;
}

/*=========================================================================*/
/* Function: SetInstrToVISABaudrate                                        */
/* Purpose:  Searches for the instrument baudrate and when found instructs */
/*           the instrument to use the VISA baudrate.                      */
/*=========================================================================*/
ViStatus rsamiq_SetInstrToVISABaudrate (ViSession instrSession)
{
	ViStatus        rsamiq_status = VI_SUCCESS;
	ViUInt32        br[] = {115200, 57600, 38400, 19200, 9600, 4800, 2400, 1200, 300},
					tmo;
	ViInt32         rate,
					i;
	ViChar          buf[100];
	ViBoolean       termCharEn;
	ViUInt8         termCh;
	ViUInt16        asrlEndIn, fCtrl,
					asrlEndOut;
	ViInt32         hdSleeps = 0;   
	ViBoolean       loopDone = 0;
	
	CHECKERR( viGetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, &termCharEn));
	CHECKERR( viGetAttribute (instrSession, VI_ATTR_TERMCHAR, &termCh));
	CHECKERR( viGetAttribute (instrSession, VI_ATTR_ASRL_END_IN, &asrlEndIn));
	CHECKERR( viGetAttribute (instrSession, VI_ATTR_ASRL_END_OUT, &asrlEndOut));
	CHECKERR( viGetAttribute (instrSession, VI_ATTR_TMO_VALUE, &tmo));
	CHECKERR( viGetAttribute (instrSession, VI_ATTR_ASRL_BAUD, &rate));
	CHECKERR( viGetAttribute (instrSession, VI_ATTR_ASRL_FLOW_CNTRL, &fCtrl));
	
#ifdef _CVI_    
	bole = SetBreakOnLibraryErrors (0);
#endif  

	CHECKERR( viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, 500));
	CHECKERR( viSetAttribute (instrSession, VI_ATTR_ASRL_END_OUT, VI_ASRL_END_NONE));

	while (!loopDone) {
		for (i=0; i<9; i++) {
			CHECKERR( viSetAttribute (instrSession, VI_ATTR_ASRL_BAUD, br[i]));
			CHECKERR( viWrite (instrSession, "\n*IDN?\n", 7, 0));
			rsamiq_Wait (hdSleeps ? 3.0 : 0.5);
			if (viRead (instrSession, (ViPBuf)buf, 100, 0) != VI_ERROR_TMO) {
				loopDone = 1;
				break;
			}   
			CHECKERR( viSetAttribute (instrSession, VI_ATTR_ASRL_END_OUT, VI_ASRL_END_BREAK));
			rsamiq_Wait (0.2);
			CHECKERR( viWrite (instrSession, "*RST\n\n", 6, 0));
			CHECKERR( viSetAttribute (instrSession, VI_ATTR_ASRL_END_OUT, VI_ASRL_END_NONE));
			rsamiq_Wait (0.2);
		}   
		if (i == 9) {
			if (hdSleeps == 0) {
				hdSleeps = 1;
			} else {
				return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
			}
		}
	}

	rsamiq_Wait (hdSleeps ? 3.0 : 0.5);
	CHECKERR( viPrintf (instrSession, ":SYST:COMM:SER:BAUD %d\n", rate));
	rsamiq_Wait (0.5);
	
	CHECKERR( viSetAttribute (instrSession, VI_ATTR_TERMCHAR_EN, termCharEn));
	CHECKERR( viSetAttribute (instrSession, VI_ATTR_TERMCHAR, termCh));
	CHECKERR( viSetAttribute (instrSession, VI_ATTR_ASRL_END_IN, asrlEndIn));
	CHECKERR( viSetAttribute (instrSession, VI_ATTR_ASRL_END_OUT, asrlEndOut));
	CHECKERR( viSetAttribute (instrSession, VI_ATTR_TMO_VALUE, tmo));
	CHECKERR( viSetAttribute (instrSession, VI_ATTR_ASRL_BAUD, rate));
	CHECKERR( viSetAttribute (instrSession, VI_ATTR_ASRL_FLOW_CNTRL, fCtrl));

#ifdef _CVI_    
	SetBreakOnLibraryErrors (bole);
#endif

	return rsamiq_status;
}   

/*=========================================================================*/
/* Function: Wait                                                          */
/* Purpose:  This function synchronously waits a given number of seconds.  */
/*=========================================================================*/
void rsamiq_Wait (ViReal64 timeToWait)
{
	ViReal64 startTime;

	if (timeToWait < 0.0)
		return;
	
	timeToWait *= CLOCKS_PER_SEC;
	
	startTime = clock ();
	
	while (clock () - startTime < timeToWait);  
		
	return;
}

/*****************************************************************************/
/*=== END INSTRUMENT DRIVER SOURCE CODE =====================================*/
/*****************************************************************************/
